@@ -4,13 +4,12 @@ from math import ceil
 from typing import Any, Callable, Dict, List
 
 import vapoursynth as vs
-from vsutil import (depth, fallback, get_depth, get_w, get_y, join, plane,
-                    scale_value)
+from vskernels import Bicubic, Box, Catrom, Point
+from vsutil import depth, fallback, get_depth, get_w, get_y, join, plane, scale_value
+from vsrgtools import repair
 
-from .kernels import Bicubic, Catrom, Point
 from .scale import ssim_downsample
-from .util import (check_variable, check_variable_format, pick_repair,
-                   scale_thresh)
+from .util import check_variable, check_variable_format, scale_thresh
 
 core = vs.core
 
@@ -29,24 +28,22 @@ __all__: List[str] = [
 def clamp_aa(src: vs.VideoNode, weak: vs.VideoNode, strong: vs.VideoNode, strength: float = 1) -> vs.VideoNode:
     """
     Clamp stronger AAs to weaker AAs.
-    Useful for clamping upscaled_sraa or eedi3 to nnedi3 for a strong but precise AA.
 
-    Stolen from Zastin.
+    Useful for clamping :py:func:`lvsfunc.aa.upscaled_sraa` or :py:func:`lvsfunc.aa.eedi3`
+    to :py:func:`lvsfunc.aa.nnedi3` for a strong but more precise AA.
 
-    :param src:      Non-AA'd source clip.
-    :param weak:     Weakly-AA'd clip (eg: nnedi3)
-    :param strong:   Strongly-AA'd clip (eg: eedi3)
-    :param strength: Clamping strength (Default: 1)
+    Original function written by `Zastin <https://github.com/kgrabs>`_, modified by LightArrowsEXE.
 
-    :return:         Clip with clamped anti-aliasing.
+    :param src:         Non-AA'd source clip.
+    :param weak:        Weakly-AA'd clip (eg: :py:func:`lvsfunc.aa.nnedi3`).
+    :param strong:      Strongly-AA'd clip (eg: :py:func:`lvsfunc.aa.eedi3`).
+    :param strength:    Clamping strength (Default: 1).
+
+    :return:            Clip with clamped anti-aliasing.
     """
-    check_variable_format(src, "clamp_aa")
-    check_variable_format(weak, "clamp_aa")
-    check_variable_format(strong, "clamp_aa")
-
-    assert src.format
-    assert weak.format
-    assert strong.format
+    assert check_variable_format(src, "clamp_aa")
+    assert check_variable_format(weak, "clamp_aa")
+    assert check_variable_format(strong, "clamp_aa")
 
     thr = strength * (1 << (src.format.bits_per_sample - 8)) if src.format.sample_type == vs.INTEGER \
         else strength/219
@@ -60,15 +57,15 @@ def clamp_aa(src: vs.VideoNode, weak: vs.VideoNode, strong: vs.VideoNode, streng
 def taa(clip: vs.VideoNode, aafun: Callable[[vs.VideoNode], vs.VideoNode]) -> vs.VideoNode:
     """
     Perform transpose AA.
+
     Example for nnedi3cl: taa(clip, nnedi3(opencl=True))
 
-    :param clip:   Input clip.
-    :param aafun:  Antialiasing function
+    :param clip:        Clip to process.
+    :param aafun:       Antialiasing function.
 
-    :return:       Antialiased clip
+    :return:            Antialiased clip.
     """
-    check_variable(clip, "taa")
-    assert clip.format
+    assert check_variable(clip, "taa")
 
     y = get_y(clip)
 
@@ -87,13 +84,13 @@ def nnedi3(opencl: bool = False, **override: Any) -> Callable[[vs.VideoNode], vs
 
     Dependencies:
 
-    * vapoursynth-nnedi3
-    * vapoursynth-NNEDI3CL (Optional: opencl)
+    * `VapourSynth-nnedi3 <https://github.com/dubhater/VapourSynth-nnedi3>`_
+    * `VapourSynth-NNEDI3CL <https://github.com/HomeOfVapourSynthEvolution/VapourSynth-NNEDI3CL>`_
 
-    :param opencl:   Use OpenCL (Default: False)
-    :param override: nnedi3 parameter overrides
+    :param opencl:          Use OpenCL (Default: False).
+    :param override:        nnedi3 parameter overrides.
 
-    :return:         Configured nnedi3 function
+    :return:                Configured nnedi3 function.
     """
     nnedi3_args: Dict[str, Any] = dict(field=0, dh=True, nsize=3, nns=3, qual=1)
     nnedi3_args.update(override)
@@ -111,12 +108,12 @@ def eedi3(opencl: bool = False, **override: Any) -> Callable[[vs.VideoNode], vs.
 
     Dependencies:
 
-    * vapoursynth-EEDI3
+    * `VapourSynth-EEDI3 <https://github.com/HomeOfVapourSynthEvolution/VapourSynth-EEDI3>`_
 
-    :param opencl:   Use OpenCL (Default: False)
-    :param override: eedi3 parameter overrides
+    :param opencl:          Use OpenCL (Default: False).
+    :param override:        eedi3 parameter overrides.
 
-    :return:         Configured eedi3 function
+    :return:                Configured eedi3 function.
     """
     eedi3_args: Dict[str, Any] = dict(field=0, dh=True, alpha=0.25, beta=0.5, gamma=40, nrad=2, mdis=20)
     eedi3_args.update(override)
@@ -132,22 +129,22 @@ def nneedi3_clamp(clip: vs.VideoNode, strength: float = 1,
                   mask: vs.VideoNode | None = None,
                   mthr: float = 0.25, opencl: bool = False) -> vs.VideoNode:
     """
-    A function that clamps eedi3 to nnedi3 for the purpose of reducing eedi3 artifacts.
-    This should fix every issue created by eedi3. For example, `see this image <https://i.imgur.com/hYVhetS.jpg>`_.
+    Clamp eedi3 to nnedi3 for the purpose of reducing eedi3 artifacts.
 
-    Original function written by Zastin, modified by LightArrowsEXE.
+    This should fix `every issue created by eedi3 <https://i.imgur.com/hYVhetS.jpg>`_.
 
-    :param clip:                Input clip
-    :param strength:            Set threshold strength for over/underflow value for clamping eedi3's result
+    Original function written by `Zastin <https://github.com/kgrabs>`_, modified by LightArrowsEXE.
+
+    :param clip:                Clip to process.
+    :param strength:            Set threshold strength for over/underflow value for clamping eedi3's result.
                                 to nnedi3 +/- strength * 256 scaled to 8 bit (Default: 1)
-    :param mask:                Clip to use for custom mask (Default: None)
-    :param mthr:                Binarize threshold for the mask, scaled to float (Default: 0.25)
-    :param opencl:              OpenCL acceleration (Default: False)
+    :param mask:                Clip to use for custom mask (Default: None).
+    :param mthr:                Binarize threshold for the mask, scaled to float (Default: 0.25).
+    :param opencl:              OpenCL acceleration (Default: False).
 
-    :return:                    Antialiased clip
+    :return:                    Antialiased clip.
     """
-    check_variable(clip, "nneedi3_clamp")
-    assert clip.format
+    assert check_variable(clip, "nneedi3_clamp")
 
     y = get_y(clip)
     mask = mask or y.std.Prewitt().std.Binarize(scale_thresh(mthr, y)).std.Maximum().std.Convolution([1]*9)
@@ -161,27 +158,27 @@ def transpose_aa(clip: vs.VideoNode,
                  eedi3: bool = False,
                  rep: int = 13) -> vs.VideoNode:
     """
-    Function that performs anti-aliasing over a clip by using nnedi3/eedi3 and transposing multiple times.
-    This results in overall stronger anti-aliasing.
-    Useful for shows like Yuru Camp with bad lineart problems.
+    Tranpose and aa a clip multiple times using nnedi3/eedi3.
 
-    Original function written by Zastin, modified by LightArrowsEXE.
+    This results in overall stronger anti-aliasing.
+    Useful for shows like Yuru Camp with bad line-art problems.
+
+    Original function written by `Zastin <https://github.com/kgrabs>`_, modified by LightArrowsEXE.
 
     Dependencies:
 
-    * RGSF (optional: 32 bit clip)
-    * vapoursynth-EEDI3
-    * vapoursynth-nnedi3
-    * znedi3
+    * `RGSF <https://github.com/IFeelBloated/RGSF>`_ (optional: 32 bit clip)
+    * `VapourSynth-EEDI3 <https://github.com/HomeOfVapourSynthEvolution/VapourSynth-EEDI3>`_
+    * `VapourSynth-nnedi3 <https://github.com/dubhater/VapourSynth-nnedi3>`_
+    * `znedi3 <https://github.com/sekrit-twc/znedi3>`_
 
-    :param clip:      Input clip
-    :param eedi3:     Use eedi3 for the interpolation (Default: False)
-    :param rep:       Repair mode. Pass it 0 to not repair (Default: 13)
+    :param clip:      Clip to process.
+    :param eedi3:     Use eedi3 for the interpolation (Default: False).
+    :param rep:       Repair mode. Pass it 0 to not repair (Default: 13).
 
-    :return:          Antialiased clip
+    :return:          Antialiased clip.
     """
-    check_variable(clip, "transpose_aa")
-    assert clip.format
+    assert check_variable(clip, "transpose_aa")
 
     clip_y = get_y(clip)
 
@@ -201,13 +198,14 @@ def transpose_aa(clip: vs.VideoNode,
 
     aaclip = _taa(clip_y)
     aaclip = _csharp(aaclip, clip_y)
-    aaclip = pick_repair(clip_y)(aaclip, clip_y, rep)
+    aaclip = repair(aaclip, clip_y, rep)
 
-    return aaclip if clip.format.color_family is vs.GRAY else core.std.ShufflePlanes([aaclip, clip], [0, 1, 2], vs.YUV)
+    return aaclip if clip.format.color_family is vs.GRAY \
+        else core.std.ShufflePlanes([aaclip, clip], [0, 1, 2], vs.YUV)
 
 
 def _nnedi3_supersample(clip: vs.VideoNode, width: int, height: int, opencl: bool = False) -> vs.VideoNode:
-    check_variable(clip, "_nnedi3_supersample")
+    assert check_variable(clip, "_nnedi3_supersample")
 
     nnargs: Dict[str, Any] = dict(field=0, dh=True, nsize=0, nns=4, qual=2)
     _nnedi3 = nnedi3(opencl=opencl, **nnargs)
@@ -219,7 +217,7 @@ def _nnedi3_supersample(clip: vs.VideoNode, width: int, height: int, opencl: boo
 
 
 def _eedi3_singlerate(clip: vs.VideoNode) -> vs.VideoNode:
-    check_variable(clip, "_eedi3_singlerate")
+    assert check_variable(clip, "_eedi3_singlerate")
 
     eeargs: Dict[str, Any] = dict(field=0, dh=False, alpha=0.2, beta=0.6, gamma=40, nrad=2, mdis=20)
     nnargs: Dict[str, Any] = dict(field=0, dh=False, nsize=0, nns=4, qual=2)
@@ -235,36 +233,36 @@ def upscaled_sraa(clip: vs.VideoNode,
                   = Bicubic(b=0, c=1/2).scale,
                   aafun: Callable[[vs.VideoNode], vs.VideoNode] = _eedi3_singlerate) -> vs.VideoNode:
     """
-    A function that performs a supersampled single-rate AA to deal with heavy aliasing and broken-up lineart.
-    Useful for heavy antialiasing.
+    Super-sampled single-rate AA for heavy aliasing and broken line-art.
 
-    It works by supersampling the clip, performing AA, and then downscaling again.
-    Downscaling can be disabled by setting `downscaler` to `None`, returning the supersampled luma clip.
+    It works by super-sampling the clip, performing AA, and then downscaling again.
+    Downscaling can be disabled by setting `downscaler` to `None`, returning the super-sampled luma clip.
     The dimensions of the downscaled clip can also be adjusted by setting `height` or `width`.
     Setting either `height` or `width` will also scale the chroma accordingly.
 
-    Original function written by Zastin, heavily modified by LightArrowsEXE.
+    Original function written by `Zastin <https://github.com/kgrabs>`_, modified by LightArrowsEXE.
 
-    Alias for this function is `lvsfunc.sraa`.
+    Alias for this function is ``lvsfunc.sraa``.
 
     Dependencies:
 
-    * vapoursynth-eedi3 (default aafun)
-    * vapoursynth-nnedi3 (default supersampler and aafun)
+    * `VapourSynth-EEDI3 <https://github.com/HomeOfVapourSynthEvolution/VapourSynth-EEDI3>`_ (default aafun)
+    * `VapourSynth-nnedi3 <https://github.com/dubhater/VapourSynth-nnedi3>`_ (default super-sampler and aafun)
 
-    :param clip:            Input clip
-    :param rfactor:         Image enlargement factor. 1.3..2 makes it comparable in strength to vsTAAmbk
+    :param clip:            Clip to process.
+    :param rfactor:         Image enlargement factor. 1.3..2 makes it comparable in strength to vsTAAmbk.
                             It is not recommended to go below 1.3 (Default: 1.5)
-    :param width:           Target resolution width. If None, determined from `height`
-    :param height:          Target resolution height (Default: ``clip.height``)
-    :param supersampler:    Supersampler used for upscaling before AA (Default: nnedi3 supersampler)
-    :param downscaler:      Downscaler to use after supersampling (Default: Bicubic(b=0, c=1/2)
-    :param aafun:           Function used to antialias after supersampling (Default: eedi3 with nnedi3 sclip)
+    :param width:           Target resolution width. If None, determined from `height`.
+    :param height:          Target resolution height (Default: ``clip.height``).
+    :param supersampler:    Super-sampler used for upscaling before AA (Default: nnedi3 super-sampler).
+    :param downscaler:      Downscaler to use after super-sampling (Default: Bicubic(b=0, c=1/2).
+    :param aafun:           Function used to antialias after super-sampling (Default: eedi3 with nnedi3 sclip).
 
-    :return:                Antialiased clip
+    :return:                Antialiased clip.
+
+    :raises ValueError:     ``rfactor`` is not above 1.
     """
-    check_variable(clip, "upscaled_sraa")
-    assert clip.format
+    assert check_variable(clip, "upscaled_sraa")
 
     luma = get_y(clip)
 
@@ -298,27 +296,31 @@ def based_aa(clip: vs.VideoNode, shader_file: str = "FSRCNNX_x2_56-16-4-1.glsl",
              mask_thr: float = 60, show_mask: bool = False,
              lmask: vs.VideoNode | None = None, **eedi3_args: Any) -> vs.VideoNode:
     """
-    As the name implies, this is a based anti-aliaser. Thank you, based Zastin.
-    This relies on FSRCNNX being very sharp, and as such it very much acts like the main "AA" here.
+    Based anti-aliaser written by the based `Zastin <https://github.com/kgrabs>`_.
 
-    Original function by Zastin, modified by LightArrowsEXE.
+    This function relies on FSRCNNX being very sharp,
+    and as such it very much acts like the main "AA" here.
+
+    Original function written by `Zastin <https://github.com/kgrabs>`_, modified by LightArrowsEXE.
 
     Dependencies:
 
-    * vapoursynth-eedi3
-    * vs-placebo
+    * `VapourSynth-EEDI3 <https://github.com/HomeOfVapourSynthEvolution/VapourSynth-EEDI3>`_
+    * `vs-placebo <https://github.com/Lypheo/vs-placebo>`_
 
-    :param clip:            Input clip
-    :param shader_file:     Path to FSRCNNX shader file
-    :param rfactor:         Image enlargement factor
-    :param tff:             Top-Field-First if true, Bottom-Field-First if false
+    :param clip:            Clip to process.
+    :param shader_file:     Path to FSRCNNX shader file.
+    :param rfactor:         Image enlargement factor.
+    :param tff:             Top-Field-First if true, Bottom-Field-First if false.
     :param mask_thr:        Threshold for the edge mask binarisation.
                             Scaled internally to match bitdepth of clip.
-    :param show_mask:       Output mask
-    :param eedi3_args:      Additional args to pass to eedi3
-    :param lmask:           Line mask clip to use for eedi3
+    :param show_mask:       Output mask.
+    :param eedi3_args:      Additional args to pass to eedi3.
+    :param lmask:           Line mask clip to use for eedi3.
 
-    :return:                AA'd clip or mask clip
+    :return:                AA'd clip or mask clip.
+
+    :raises ValueError:     ``l_mask`` is passed and '`mask_thr` is higher than 255.
     """
     def _eedi3s(clip: vs.VideoNode, mclip: vs.VideoNode | None = None,
                 **eedi3_kwargs: Any) -> vs.VideoNode:
@@ -344,10 +346,10 @@ def based_aa(clip: vs.VideoNode, shader_file: str = "FSRCNNX_x2_56-16-4-1.glsl",
 
         if (ow > iw and ow/iw != ow//iw) or (oh > ih and oh/ih != oh//ih):
             mclip = Point().scale(mclip, iw * ceil(ow / iw), ih * ceil(oh / ih))
-        return core.fmtc.resample(mclip, ow, oh, kernel='box', fulls=1, fulld=1)
 
-    check_variable(clip, "based_aa")
-    assert clip.format
+        return Box(fulls=1, fulld=1).scale(mclip, ow, oh)
+
+    assert check_variable(clip, "based_aa")
 
     aaw = (round(clip.width * rfactor) + 1) & ~1
     aah = (round(clip.height * rfactor) + 1) & ~1
