@@ -5,21 +5,18 @@ import random
 import warnings
 from abc import ABC, abstractmethod
 from itertools import groupby, zip_longest
-from typing import (Any, Callable, Dict, Iterable, Iterator, List, Literal,
-                    Sequence, Set, Tuple, TypeVar, overload)
+from typing import Any, Callable, Dict, Iterable, Iterator, List, Literal, Sequence, Set, Tuple, TypeVar, overload
 
 import vapoursynth as vs
+from vskernels import Catrom, get_matrix
 from vsutil import depth, get_subsampling, get_w
 from vsutil import split as split_planes
 
 from .dehardsub import hardsub_mask
-from .exceptions import (ClipsAndNamedClipsError, InvalidFormatError,
-                         NotEqualFormatsError, VariableFormatError)
-from .kernels import Catrom
+from .exceptions import ClipsAndNamedClipsError, InvalidFormatError, NotEqualFormatsError, VariableFormatError
 from .render import clip_async_render
 from .types import Direction
-from .util import (check_variable, check_variable_format,
-                   check_variable_resolution, get_matrix, get_prop)
+from .util import check_variable, check_variable_format, check_variable_resolution, get_prop
 
 core = vs.core
 
@@ -48,16 +45,22 @@ class Comparer(ABC):
     """
     Base class for comparison functions.
 
-    :param clips:           A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
-                            If given a dict, the names will be overlaid on the clips using
-                            ``VideoNode.text.Text``.
-                            If given a simple sequence of clips, the `label_alignment` parameter will have no effect
-                            and the clips will not be labeled.
-                            The order of the clips in either a dict or a sequence will be kept in the comparison.
-    :param label_alignment: An integer from 1-9, corresponding to the positions of the keys on a numpad.
-                            Only used if `clips` is a dict.
-                            Determines where to place clip name using ``VideoNode.text.Text`` (Default: 7)
+    :param clips:               A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
+                                If given a dict, the names will be overlaid on the clips
+                                using :py:func:`vapoursynth.core.text.Text`.
+                                If given a simple sequence of clips, the `label_alignment` parameter will have no effect
+                                and the clips will not be labeled.
+                                The order of the clips in either a dict or a sequence will be kept in the comparison.
+    :param label_alignment:     An integer from 1-9, corresponding to the positions of the keys on a numpad.
+                                Only used if `clips` is a dict.
+                                Determines where to place clip name using :py:func:`vapoursynth.core.text.Text`
+                                (Default: 7).
+
+    :raises ValueError:         Less than two clips passed.
+    :raises ValueError:         `label_alignment` is not between 1–9.
+    :raises ValueError:         Unexpected type passed to `clips`.
     """
+
     def __init__(self,
                  clips: Dict[str, vs.VideoNode] | Sequence[vs.VideoNode],
                  /,
@@ -88,9 +91,7 @@ class Comparer(ABC):
         self.format: str | None = formats.pop() if len(formats) == 1 else None
 
     def _marked_clips(self) -> List[vs.VideoNode]:
-        """
-        If a `name` is only space characters, `'   '`, for example, the name will not be overlaid on the clip.
-        """
+        """If a `name` is only space characters, `'   '`, for example, the name will not be overlaid on the clip."""
         if self.names:
             return [clip.text.Text(text=name, alignment=self.label_alignment) if name.strip() else clip
                     for clip, name in zip(self.clips, self.names)]
@@ -104,7 +105,7 @@ class Comparer(ABC):
     @property
     def clip(self) -> vs.VideoNode:
         """
-        Returns the comparison as a single VideoNode for further manipulation or attribute inspection.
+        Return the comparison as a single VideoNode for further manipulation or attribute inspection.
 
         ``comp_clip = Comparer(...).clip`` is the intended use in encoding scripts.
         """
@@ -115,19 +116,23 @@ class Stack(Comparer):
     """
     Stacks clips horizontally or vertically.
 
-    Acts as a convenience combination function of ``vapoursynth.core.text.Text``
-    and either ``vapoursynth.core.std.StackHorizontal`` or ``vapoursynth.core.std.StackVertical``.
+    Acts as a convenience combination function of :py:func:`vapoursynth.core.text.Text`
+    and either :py:func:`vapoursynth.core.std.StackHorizontal` or :py:func:`vapoursynth.core.std.StackVertical`.
 
-    :param clips:           A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
-                            If given a dict, the names will be overlaid on the clips using
-                            ``VideoNode.text.Text``.
-                            If given a simple sequence of clips, the `label_alignment` parameter will have no effect
-                            and the clips will not be labeled.
-                            The order of the clips in either a dict or a sequence will be kept in the comparison.
-    :param direction:       Direction of the stack (Default: :py:attr:`lvsfunc.comparison.Direction.HORIZONTAL`)
-    :param label_alignment: An integer from 1-9, corresponding to the positions of the keys on a numpad.
-                            Only used if `clips` is a dict.
-                            Determines where to place clip name using ``VideoNode.text.Text`` (Default: 7)
+    :param clips:               A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
+                                If given a dict, the names will be overlaid on the clips using
+                                :py:func:`vapoursynth.core.text.Text`.
+                                If given a simple sequence of clips, the `label_alignment` parameter will have no effect
+                                and the clips will not be labeled.
+                                The order of the clips in either a dict or a sequence will be kept in the comparison.
+    :param direction:           Direction of the stack (Default: :py:attr:`lvsfunc.comparison.Direction.HORIZONTAL`).
+    :param label_alignment:     An integer from 1-9, corresponding to the positions of the keys on a numpad.
+                                Only used if `clips` is a dict.
+                                Determines where to place clip name using :py:func:`vapoursynth.core.text.Text`
+                                (Default: 7).
+
+    :raises ValueError:         Clips are not the same height for StackHorizontal.
+    :raises ValueError:         Clips are not the same width for StackVertical.
     """
 
     def __init__(self,
@@ -153,21 +158,23 @@ class Stack(Comparer):
 
 class Interleave(Comparer):
     """
-    From the VapourSynth documentation: `Returns a clip with the frames from all clips interleaved.
-    For example, Interleave(A=clip1, B=clip2) will return A.Frame 0, B.Frame 0, A.Frame 1, B.Frame 1, ...`
+    Returns a clip with the frames from all clips interleaved.
 
-    Acts as a convenience combination function of ``vapoursynth.core.text.Text``
-    and ``vapoursynth.core.std.Interleave``.
+    For example, Interleave(A=clip1, B=clip2) will return A.Frame 0, B.Frame 0, A.Frame 1, B.Frame 1, ...
 
-    :param clips:           A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
-                            If given a dict, the names will be overlaid on the clips using
-                            ``VideoNode.text.Text``.
-                            If given a simple sequence of clips, the `label_alignment` parameter will have no effect
-                            and the clips will not be labeled.
-                            The order of the clips in either a dict or a sequence will be kept in the comparison.
-    :param label_alignment: An integer from 1-9, corresponding to the positions of the keys on a numpad.
-                            Only used if `clips` is a dict.
-                            Determines where to place clip name using ``VideoNode.text.Text`` (Default: 7)
+    Acts as a convenience combination function of :py:func:`vapoursynth.core.text.Text`
+    and :py:func:`vapoursynth.core.std.Interleave`.
+
+    :param clips:               A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
+                                If given a dict, the names will be overlaid on the clips using
+                                :py:func:`vapoursynth.core.text.Text`.
+                                If given a simple sequence of clips, the `label_alignment` parameter will have no effect
+                                and the clips will not be labeled.
+                                The order of the clips in either a dict or a sequence will be kept in the comparison.
+    :param label_alignment:     An integer from 1-9, corresponding to the positions of the keys on a numpad.
+                                Only used if `clips` is a dict.
+                                Determines where to place clip name using :py:func:`vapoursynth.core.text.Text`
+                                (Default: 7).
     """
 
     def __init__(self,
@@ -215,17 +222,23 @@ class Tile(Comparer):
         ]
         # the rows will be auto-padded with 0's to be the same length.
 
-    :param clips:           A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
-                            If given a dict, the names will be overlaid on the clips using
-                            ``VideoNode.text.Text``.
-                            If given a simple sequence of clips, the `label_alignment` parameter will have no effect
-                            and the clips will not be labeled.
-                            The order of the clips in either a dict or a sequence will be kept in the comparison.
-    :param arrangement:     2-dimension array (list of lists) of 0s and 1s representing a list of rows of clips(`1`)
-                            or blank spaces(`0`) (Default: ``None``)
-    :param label_alignment: An integer from 1-9, corresponding to the positions of the keys on a numpad.
-                            Only used if `clips` is a dict.
-                            Determines where to place clip name using ``VideoNode.text.Text`` (Default: 7)
+    :param clips:               A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
+                                If given a dict, the names will be overlaid on the clips using
+                                :py:func:`vapoursynth.core.text.Text`.
+                                If given a simple sequence of clips, the `label_alignment` parameter will have no effect
+                                and the clips will not be labeled.
+                                The order of the clips in either a dict or a sequence will be kept in the comparison.
+    :param arrangement:         2-dimension array (list of lists) of 0s and 1s
+                                representing a list of rows of clips(`1`) or blank spaces(`0`) (Default: ``None``)
+    :param label_alignment:     An integer from 1-9, corresponding to the positions of the keys on a numpad.
+                                Only used if `clips` is a dict.
+                                Determines where to place clip name using :py:func:`vapoursynth.core.text.Text`
+                                (Default: 7).
+
+    :raises ValueError:         Clip heights and widths don't match.
+    :raises ValueError:         Array is one-dimensional and you should be using
+                                :py:class:`lvsfunc.comparison.Stack` instead.
+    :raises ValueError:         Specified arrangement has an invalid number of clips.
     """
 
     def __init__(self,
@@ -276,23 +289,28 @@ class Tile(Comparer):
 class Split(Stack):
     """
     Split an unlimited amount of clips into one VideoNode with the same dimensions as the original clips.
+
     Handles odd-sized resolutions or resolutions that can't be evenly split by the amount of clips specified.
 
     The remaining pixel width/height (``clip.dimension % number_of_clips``)
     will be always given to the last clip specified.
-    For example, five `104 x 200` clips will result in a `((20 x 200) * 4) + (24 x 200)` horiztonal stack of clips.
+    For example, five `104 x 200` clips will result in a `((20 x 200) * 4) + (24 x 200)` horizontal stack of clips.
 
-    :param clips:           A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
-                            If given a dict, the names will be overlaid on the clips using
-                            ``VideoNode.text.Text``.
-                            If given a simple sequence of clips, the `label_alignment` parameter will have no effect
-                            and the clips will not be labeled.
-                            The order of the clips in either a dict or a sequence will be kept in the comparison.
-    :param direction:       Determines the axis to split the clips on
-                            (Default: :py:attr:`lvsfunc.comparison.Direction.HORIZONTAL`)
-    :param label_alignment: An integer from 1-9, corresponding to the positions of the keys on a numpad.
-                            Only used if `clips` is a dict.
-                            Determines where to place clip name using ``VideoNode.text.Text`` (Default: 7)
+    :param clips:               A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
+                                If given a dict, the names will be overlaid on the clips using
+                                :py:func:`vapoursynth.core.text.Text`.
+                                If given a simple sequence of clips, the `label_alignment` parameter will have no effect
+                                and the clips will not be labeled.
+                                The order of the clips in either a dict or a sequence will be kept in the comparison.
+    :param direction:           Determines the axis to split the clips on.
+                                (Default: :py:attr:`lvsfunc.comparison.Direction.HORIZONTAL`)
+    :param label_alignment:     An integer from 1-9, corresponding to the positions of the keys on a numpad.
+                                Only used if `clips` is a dict.
+                                Determines where to place clip name using :py:func:`vapoursynth.core.text.Text`
+                                (Default: 7).
+
+    :raises ValueError:         Clip heights and widths don't match.
+    :raises ValueError:         Resulting cropped width or height violates subsampling rules.
     """
 
     def __init__(self,
@@ -352,24 +370,29 @@ def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
             force_resample: bool = True, print_frame: bool = True,
             mismatch: bool = False) -> vs.VideoNode:
     """
-    Allows for the same frames from two different clips to be compared by interleaving them into a single clip.
+    Compare the same frames from two different clips by interleaving them into a single clip.
+
     Clips are automatically resampled to 8 bit YUV -> RGB24 to emulate how a monitor shows the frame.
     This can be disabled by setting `force_resample` to ``False``.
 
     This is not recommended over setting multiple outputs and checking between those,
     but in the event that is unavailable to you, this function may be useful.
 
-    Alias for this function is `lvsfunc.comp`.
+    Alias for this function is ``lvsfunc.comp``.
 
-    :param clip_a:         Clip to compare
-    :param clip_b:         Second clip to compare
-    :param frames:         List of frames to compare (Default: ``None``)
-    :param rand_total:     Number of random frames to pick (Default: ``None``)
-    :param force_resample: Forcibly resamples the clip to RGB24 (Default: ``True``)
-    :param print_frame:    Print frame numbers (Default: ``True``)
-    :param mismatch:       Allow for clips with different formats and dimensions to be compared (Default: ``False``)
+    :param clip_a:          Clip to compare.
+    :param clip_b:          Second clip to compare.
+    :param frames:          List of frames to compare (Default: ``None``).
+    :param rand_total:      Number of random frames to pick (Default: ``None``).
+    :param force_resample:  Forcibly resamples the clip to RGB24 (Default: ``True``).
+    :param print_frame:     Print frame numbers (Default: ``True``).
+    :param mismatch:        Allow for clips with different formats and dimensions to be compared (Default: ``False``).
 
-    :return:               Interleaved clip containing specified frames from `clip_a` and `clip_b`
+    :return:                Interleaved clip containing specified frames from `clip_a` and `clip_b`.
+
+    :raises ValueError:     More comparisons requested than frames available.
+    :raises NotEqualFormatsError:
+                            Format of given clips don't match.
     """
     def _resample(clip: vs.VideoNode) -> vs.VideoNode:
         # Resampling to 8 bit and RGB to properly display how it appears on your screen
@@ -386,11 +409,8 @@ def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
     if force_resample:
         clip_a, clip_b = _resample(clip_a), _resample(clip_b)
     elif mismatch is False:
-        check_variable_format(clip_a, "compare")
-        check_variable_format(clip_b, "compare")
-
-        assert clip_a.format
-        assert clip_b.format
+        assert check_variable_format(clip_a, "compare")
+        assert check_variable_format(clip_b, "compare")
 
         if clip_a.format.id != clip_b.format.id:
             raise NotEqualFormatsError("compare")
@@ -411,32 +431,25 @@ def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
 
 
 def stack_compare(*clips: vs.VideoNode,
-                  make_diff: bool = True,
                   height: int | None = None) -> vs.VideoNode:
     """
-    A simple wrapper that allows you to compare two clips by stacking them.
+    Compare two clips by stacking them.
 
     Best to use when trying to match two sources frame-accurately.
-    Alias for this function is `lvsfunc.scomp`.
+    Alias for this function is ``lvsfunc.scomp``.
 
-    When not using `make_diff`, `Stack` is heavily recommended instead.
 
-    :param clips:     Clips to compare
-    :param make_diff: Create and stack a diff (only works if two clips are given) (Default: ``True``)
-    :param height:    Height in px to rescale clips to if `make_diff` is ``True``
-                      (MakeDiff clip will be twice this resolution).
-                      This function will not scale above the first clip's height (Default: 288).
+    :param clips:           Clips to compare.
+    :param height:          Height in px to rescale clips to.
+                            (MakeDiff clip will be twice this resolution).
+                            This function will not scale above the first clip's height (Default: 288).
 
-    :return:          Clip with `clips` stacked
+    :return:                Clip with `clips` stacked.
+
+    :raises ValueError:     More or less than 2 clips are given.
     """
-
-    if not make_diff:
-        warnings.warn("stack_compare has been deprecated in favour of `lvsfunc.comparison.Stack` "
-                      "when not using `make_diff`", DeprecationWarning)
-        return core.std.StackHorizontal(clips)
-
     if len(clips) != 2:
-        raise ValueError("stack_compare: `make_diff` only works for exactly 2 clips")
+        raise ValueError(f"stack_compare: You must pass at least two clips, not {len(clips)}.")
 
     clipa, clipb = clips
 
@@ -460,17 +473,21 @@ def stack_compare(*clips: vs.VideoNode,
 def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.VideoNode:
     """
     Stacks the planes of a clip.
+
     For 4:2:0 subsampled clips, the two half-sized planes will be stacked in the opposite direction specified
     (vertical by default),
     then stacked with the full-sized plane in the direction specified (horizontal by default).
 
-    :param clip:           Input clip (must be in YUV or RGB planar format)
-    :param stack_vertical: Stack the planes vertically (Default: ``False``)
+    :param clip:                    Clip to process (must be in YUV or RGB planar format).
+    :param stack_vertical:          Stack the planes vertically (Default: ``False``).
 
-    :return:               Clip with stacked planes
+    :return:                        Clip with stacked planes.
+
+    :raises InvalidFormatError:     Clip is not YUV or RGB.
+    :raises TypeError:              Clip is of an unexpected color family.
+    :raises TypeError:              Clip returns an unexpected subsampling.
     """
-    check_variable(clip, "stack_planes")
-    assert clip.format
+    assert check_variable(clip, "stack_planes")
 
     if clip.format.num_planes != 3:
         raise InvalidFormatError("stack_planes", "{func}: Input clip must be of a YUV or RGB planar format!")
@@ -488,14 +505,12 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
 
     if get_subsampling(clip) in ('444', None):
         return Stack(planes, direction=direction).clip
-
     elif get_subsampling(clip) == '420':
         subsample_direction: Direction = Direction.HORIZONTAL if stack_vertical else Direction.VERTICAL
         y_plane = planes.pop('Y').text.Text(text='Y')
         subsampled_planes = Stack(planes, direction=subsample_direction).clip
 
         return Stack([y_plane, subsampled_planes], direction=direction).clip
-
     else:
         raise TypeError(f"stack_planes: Unexpected subsampling {get_subsampling(clip)}!")
 
@@ -503,15 +518,16 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
 def diff_hardsub_mask(a: vs.VideoNode, b: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
     """
     Diff func for :py:func:`lvsfunc.comparison.diff` to use a hardsub mask.
+
     This is kinda slow.
 
-    :param a: Clip A
-    :param b: Clip B
+    :param a:   Clip A.
+    :param b:   Clip B.
 
-    :return:  Diff masked with :py:func:`lvsfunc.dehardsub.hardsub_mask`
+    :return:    Diff masked with :py:func:`lvsfunc.dehardsub.hardsub_mask`.
     """
-    check_variable(a, "diff_hardsub_mask")
-    check_variable(b, "diff_hardsub_mask")
+    assert check_variable(a, "diff_hardsub_mask")
+    assert check_variable(b, "diff_hardsub_mask")
 
     return core.std.MaskedMerge(core.std.MakeDiff(a, a), core.std.MakeDiff(a, b), hardsub_mask(a, b, **kwargs))
 
@@ -541,7 +557,7 @@ def diff(*clips: vs.VideoNode,
 
 
 def diff(*clips: vs.VideoNode,
-         thr: float = 72,
+         thr: float = 96,
          height: int = 288,
          interleave: bool = False,
          return_ranges: bool = False,
@@ -549,7 +565,8 @@ def diff(*clips: vs.VideoNode,
          diff_func: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] = lambda a, b: core.std.MakeDiff(a, b),
          **namedclips: vs.VideoNode) -> vs.VideoNode | Tuple[vs.VideoNode, List[Tuple[int, int]]]:
     """
-    Creates a standard :py:class:`lvsfunc.comparison.Stack` between frames from two clips that have differences.
+    Create a standard :py:class:`lvsfunc.comparison.Stack` between frames from two clips that have differences.
+
     Useful for making comparisons between TV and BD encodes, as well as clean and hardsubbed sources.
 
     There are two methods used here to find differences:
@@ -562,26 +579,34 @@ def diff(*clips: vs.VideoNode,
     Note that this might catch artifacting as differences!
     Make sure you verify every frame with your own eyes!
 
-    Alias for this function is `lvsfunc.diff`.
+    Alias for this function is ``lvsfunc.diff``.
 
-    :param clips:               Clips for comparison (order is kept)
+    :param clips:               Clips for comparison (order is kept).
     :param namedclips:          Keyword arguments of `name=clip` for all clips in the comparison.
                                 Clips will be labeled at the top left with their `name`.
     :param thr:                 Threshold, <= 1 uses PlaneStatsDiff, > 1 uses Max/Min.
                                 Higher values will catch more differences.
                                 Value must be lower than 128
-    :param height:              Height in px to downscale clips to if `interleave` is ``False``
+    :param height:              Height in px to downscale clips to if `interleave` is ``False``.
                                 (MakeDiff clip will be twice this resolution)
-    :param interleave:          Return clip as an interleaved comparison
+    :param interleave:          Return clip as an interleaved comparison.
                                 (using :py:class:`lvsfunc.comparison.Interleave`).
                                 This will not return a diff clip
-    :param return_ranges:       Return a list of ranges in addition to the comparison clip
-    :param exclusion_ranges:    Excludes a list of frame ranges from difference checking output (but not processing)
-    :param diff_func:           Function for calculating diff in PlaneStatsMin/Max mode
+    :param return_ranges:       Return a list of ranges in addition to the comparison clip.
+    :param exclusion_ranges:    Excludes a list of frame ranges from difference checking output (but not processing).
+    :param diff_func:           Function for calculating diff in PlaneStatsMin/Max mode.
 
     :return:                    Either an interleaved clip of the differences between the two clips
                                 or a stack of both input clips on top of MakeDiff clip.
                                 Furthermore, the function will print the ranges of all the diffs found.
+
+    :raises ClipsAndNamedClipsError:
+                                Both positional and named clips are given.
+    :raises ValueError:         More or less than two clips are given.
+    :raises ValueError:         ``thr`` is 128 or higher.
+    :raises VariableFormatError:
+                                One of the clips is of a variable format.
+    :raises StopIteration:      No differences are found.
     """
     if clips and namedclips:
         raise ClipsAndNamedClipsError("diff")
@@ -592,14 +617,14 @@ def diff(*clips: vs.VideoNode,
     if thr >= 128:
         raise ValueError("diff: `thr` must be below 128!")
 
-    if clips and any(c.format is None for c in clips):
+    if clips and not all([c.format for c in clips]):  # noqa
         raise VariableFormatError("diff")
-    elif namedclips and any(nc.format is None for nc in namedclips.values()):
+    elif namedclips and not all([nc.format for nc in namedclips.values()]):
         raise VariableFormatError("diff")
 
     def _to_ranges(iterable: List[int]) -> Iterable[Tuple[int, int]]:
         iterable = sorted(set(iterable))
-        for key, group in groupby(enumerate(iterable), lambda t: t[1] - t[0]):
+        for _, group in groupby(enumerate(iterable), lambda t: t[1] - t[0]):
             groupl = list(group)
             yield groupl[0][1], groupl[-1][1]
 
@@ -621,8 +646,7 @@ def diff(*clips: vs.VideoNode,
         diff = core.std.MakeDiff(a, b)
     else:
         diff = diff_func(a, b).std.PlaneStats()
-        assert diff.format is not None
-        t = float if diff.format.sample_type == vs.FLOAT else int
+        t = float if diff.format.sample_type == vs.FLOAT else int  # type:ignore[union-attr]
 
         def _cb(n: int, f: vs.VideoFrame) -> None:
             if get_prop(f, 'PlaneStatsMin', t) <= thr or get_prop(f, 'PlaneStatsMax', t) >= 255 - thr > thr:
@@ -675,11 +699,14 @@ def interleave(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode
     """
     Small convenience function for interleaving clips.
 
-    :param clips:      Clips for comparison (order is kept)
-    :param namedclips: Keyword arguments of `name=clip` for all clips in the comparison.
-                       Clips will be labeled at the top left with their `name`.
+    :param clips:       Clips for comparison (order is kept).
+    :param namedclips:  Keyword arguments of `name=clip` for all clips in the comparison.
+                        Clips will be labeled at the top left with their `name`.
 
-    :return: An interleaved clip of all the `clips`/`namedclips` specified
+    :return:            An interleaved clip of all the `clips`/`namedclips` specified.
+
+    :raises ClipsAndNamedClipsError:
+                        Both positional and named clips are given.
     """
     if clips and namedclips:
         raise ClipsAndNamedClipsError("interleave")
@@ -689,15 +716,19 @@ def interleave(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode
 def split(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
     """
     Small convenience function for splitting clips along the x-axis and then stacking.
+
     Accounts for odd-resolution clips by giving overflow columns to the last clip specified.
     All clips must have the same dimensions (width and height).
 
-    :param clips:      Clips for comparison (order is kept left to right)
-    :param namedclips: Keyword arguments of `name=clip` for all clips in the comparison.
-                       Clips will be labeled at the bottom with their `name`.
+    :param clips:       Clips for comparison (order is kept left to right).
+    :param namedclips:  Keyword arguments of `name=clip` for all clips in the comparison.
+                        Clips will be labeled at the bottom with their `name`.
 
-    :return: A clip with the same dimensions as any one of the input clips
-             with all `clips`/`namedclips` represented as individual vertical slices.
+    :return:            A clip with the same dimensions as any one of the input clips.
+                        with all `clips`/`namedclips` represented as individual vertical slices.
+
+    :raises ClipsAndNamedClipsError:
+                        Both positional and named clips are given.
     """
     if clips and namedclips:
         raise ClipsAndNamedClipsError("split")
@@ -708,11 +739,14 @@ def stack_horizontal(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.Vid
     """
     Small convenience function for stacking clips horizontally.
 
-    :param clips:      Clips for comparison (order is kept left to right)
-    :param namedclips: Keyword arguments of `name=clip` for all clips in the comparison.
-                       Clips will be labeled at the top left with their `name`.
+    :param clips:       Clips for comparison (order is kept left to right).
+    :param namedclips:  Keyword arguments of `name=clip` for all clips in the comparison.
+                        Clips will be labeled at the top left with their `name`.
 
-    :return: A horizontal stack of the `clips`/`namedclips`
+    :return:            A horizontal stack of the `clips`/`namedclips`.
+
+    :raises ClipsAndNamedClipsError:
+                        Both positional and named clips are given.
     """
     if clips and namedclips:
         raise ClipsAndNamedClipsError("stack_horizontal")
@@ -723,11 +757,14 @@ def stack_vertical(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.Video
     """
     Small convenience function for stacking clips vertically.
 
-    :param clips:      Clips for comparison (order is kept top to bottom)
-    :param namedclips: Keyword arguments of `name=clip` for all clips in the comparison.
-                       Clips will be labeled at the top left with their `name`.
+    :param clips:       Clips for comparison (order is kept top to bottom).
+    :param namedclips:  Keyword arguments of `name=clip` for all clips in the comparison.
+                        Clips will be labeled at the top left with their `name`.
 
-    :return: A vertical stack of the `clips`/`namedclips`
+    :return:            A vertical stack of the `clips`/`namedclips`.
+
+    :raises ClipsAndNamedClipsError:
+                        Both positional and named clips are given.
     """
     if clips and namedclips:
         raise ClipsAndNamedClipsError("stack_vertical")
@@ -737,16 +774,20 @@ def stack_vertical(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.Video
 def tile(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
     """
     Small convenience function for tiling clips in a rectangular pattern.
+
     All clips must have the same dimensions (width and height).
     If 3 clips are given, a 2x2 square with one blank slot will be returned.
     If 6 clips are given, a 3x2 rectangle will be returned.
 
-    :param clips:      Clips for comparison
-    :param namedclips: Keyword arguments of `name=clip` for all clips in the comparison.
-                       Clips will be labeled at the top left with their `name`.
+    :param clips:       Clips for comparison.
+    :param namedclips:  Keyword arguments of `name=clip` for all clips in the comparison.
+                        Clips will be labeled at the top left with their `name`.
 
-    :return: A clip with all input `clips`/`namedclips` automatically tiled most optimally
-             into a rectangular arrrangement
+    :return:            A clip with all input `clips`/`namedclips` automatically tiled most optimally
+                        into a rectangular arrrangement.
+
+    :raises ClipsAndNamedClipsError:
+                        Both positional and named clips are given.
     """
     if clips and namedclips:
         raise ClipsAndNamedClipsError("tile")
