@@ -7,31 +7,32 @@ import warnings
 from fractions import Fraction
 from functools import partial
 from pathlib import Path
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Sequence
 
 import vapoursynth as vs
-from vsutil import Dither, depth, get_depth, get_w, get_y, scale_value
+from vskernels import Bicubic, BicubicDidee, Catrom, Kernel, get_kernel
+from vsrgtools import repair
+from vsutil import Dither, depth, get_depth, get_neutral_value, get_w, get_y, scale_value
 
 from .comparison import Stack
-from .exceptions import InvalidFramerateError
-from .kernels import Bicubic, BicubicDidee, Catrom, Kernel, get_kernel
-from .render import get_render_progress
+from .exceptions import InvalidFramerateError, TopFieldFirstError
+from .render import clip_async_render, get_render_progress
 from .types import Direction
-from .util import (check_variable, check_variable_format, force_mod,
-                   get_neutral_value, get_prop, pick_repair)
+from .util import check_variable, check_variable_format, force_mod, get_prop
 
 core = vs.core
 
 __all__: List[str] = [
+    'check_patterns',
     'deblend',
     'decomb',
     'descale_fields',
     'fix_telecined_fades',
-    'ivtc_credits',
+    'pulldown_credits', 'ivtc_credits',
     'seek_cycle',
     'sivtc', 'SIVTC',
     'tivtc_vfr', 'TIVTC_VFR',
-    'vinverse'
+    'vinverse',
 ]
 
 main_file = os.path.realpath(sys.argv[0]) if sys.argv[0] else None
@@ -42,17 +43,17 @@ main_file = "{yourScriptName}_" if main_file in ("__main___", "setup_") else mai
 def sivtc(clip: vs.VideoNode, pattern: int = 0,
           tff: bool = True, decimate: bool = True) -> vs.VideoNode:
     """
-    A very simple fieldmatching function.
+    Simplest form of a fieldmatching function.
 
     This is essentially a stripped-down JIVTC offering JUST the basic fieldmatching and decimation part.
     As such, you may need to combine multiple instances if patterns change throughout the clip.
 
-    :param clip:        Input clip
-    :param pattern:     First frame of any clean-combed-combed-clean-clean sequence
-    :param tff:         Top-Field-First
-    :param decimate:    Drop a frame every 5 frames to get down to 24000/1001
+    :param clip:        Clip to process.
+    :param pattern:     First frame of any clean-combed-combed-clean-clean sequence.
+    :param tff:         Top-Field-First.
+    :param decimate:    Drop a frame every 5 frames to get down to 24000/1001.
 
-    :return:            IVTC'd clip
+    :return:            IVTC'd clip.
     """
     pattern = pattern % 5
 
@@ -64,26 +65,32 @@ def sivtc(clip: vs.VideoNode, pattern: int = 0,
 
 def seek_cycle(clip: vs.VideoNode, write_props: bool = True, scale: int = -1) -> vs.VideoNode:
     """
-    Purely visual tool to view telecining cycles. This is purely a visual tool!
-    This function has no matching parameters, just use wobbly instead if you need that.
+    Purely visual tool to view telecining cycles.
 
-    Displays the current frame, two previous and two future frames,
-    as well as whether they are combed or not.
+    .. warning::
+        | This is purely a visual tool and has no matching parameters!
+        | Just use `Wobbly <https://github.com/dubhater/Wobbly>`_ instead if you need that.
 
-    P indicates a progressive frame, C a combed frame.
+    Displays the current frame, two previous and future frames,
+    and whether they are combed or not.
+
+    ``P`` indicates a progressive frame,
+    and ``C`` a combed frame.
 
     Dependencies:
 
-    * VapourSynth-TDeintMod
+    * `VapourSynth-TDeintMod <https://github.com/HomeOfVapourSynthEvolution/VapourSynth-TDeintMod>`_
 
-    :param clip:            Input clip
+    :param clip:            Clip to process.
     :param write_props:     Write props on frames. Disabling this will also speed up the function.
     :param scale:           Integer scaling of all clips. Must be to the power of 2.
 
-    :return:                Viewing UI for standard telecining cycles
+    :return:                Viewing UI for standard telecining cycles.
+
+    :raises ValueError:     `scale` is a value that is not to the power of 2.
     """
     if (scale & (scale-1) != 0) and scale != 0 and scale != -1:
-        raise ValueError("seek_cycle: 'scale must be a value that is the power of 2!'")
+        raise ValueError("seek_cycle: '`scale` must be a value that is the power of 2!'")
 
     # TODO: 60i checks and flags somehow? false positives gonna be a pain though
     def check_combed(n: int, f: vs.VideoFrame, clip: vs.VideoNode) -> vs.VideoNode:
@@ -124,6 +131,55 @@ def seek_cycle(clip: vs.VideoNode, write_props: bool = True, scale: int = -1) ->
     return Stack([vert_pad, stack, vert_pad, stack_abcde], direction=Direction.VERTICAL).clip
 
 
+def check_patterns(clip: vs.VideoNode, tff: bool | int | None = None) -> int:
+    """
+    Naïve function that iterates over a given clip and tries out every simple 3:2 IVTC pattern.
+
+    This function will return the best pattern value that didn't result in any combing.
+    If all of them resulted in combing, it will raise an error.
+
+    Note that the clip length may seem off because I grab every fourth frame of a clip.
+    This should make processing faster, and it will still find combed frames.
+
+    This function should only be used for rudimentary testing.
+    If I see it in any proper scripts, heads will roll.
+
+    Dependencies:
+
+    * `VapourSynth-TDeintMod <https://github.com/HomeOfVapourSynthEvolution/VapourSynth-TDeintMod>`_
+
+    :param clip:                    Clip to process.
+    :param tff:                     Top-field-first. `False` sets it to Bottom-Field-First.
+                                    If None, get the field order from the _FieldBased prop.
+
+    :return:                        Integer representing the best pattern.
+
+    :raises TopFieldFirstError:     No automatic ``tff`` can be determined.
+    :raises StopIteration:          No pattern resulted in a clean match.
+    """
+    if get_prop(clip.get_frame(0), '_FieldBased', int) == 0 and tff is None:
+        raise TopFieldFirstError("check_patterns")
+    elif isinstance(tff, (bool, int)):
+        clip = clip.std.SetFieldBased(int(tff) + 1)
+
+    clip = depth(clip, 8)
+
+    pattern = -1
+
+    for n in [int(n) for n in range(0, 4)]:
+        check = _check_pattern(clip, n)
+
+        if check:
+            pattern = n
+            break
+
+    if pattern == -1:
+        raise StopIteration("check_patterns: 'None of the patterns resulted in a clip without combing. "
+                            "Please try performing proper IVTC on the clip.")
+
+    return pattern
+
+
 def tivtc_vfr(clip: vs.VideoNode,
               tfm_in: Path | str = f".ivtc/{main_file}matches.txt",
               tdec_in: Path | str = f".ivtc/{main_file}metrics.txt",
@@ -132,21 +188,24 @@ def tivtc_vfr(clip: vs.VideoNode,
               tfm_args: Dict[str, Any] = {},
               tdecimate_args: Dict[str, Any] = {}) -> vs.VideoNode:
     """
-    Wrapper for performing TFM and TDecimate on a clip that is supposed to be VFR.
+    Perform TFM and TDecimate on a clip that is supposed to be VFR.
+
     Includes automatic generation of a metrics/matches/timecodes txt file.
 
     | This function took *heavy* inspiration from atomchtools.TIVTC_VFR,
     | and is basically an improved rewrite on the concept.
 
     .. warning::
-        | When calculating the matches and metrics for the first time, your previewer may error!
-        | To fix this, refresh your previewer! If it still doesn't work, open the ``.ivtc`` directory
-        | and check if the files in there are **0kb**. If they are, **delete them** and run the function again.
-        | You may need to first restart your previewer entirely for it to work!
+        | When calculating the matches and metrics for the first time, your previewer may error out!
+        | To fix this, simply refresh your previewer. If it still doesn't work, open the ``.ivtc`` directory
+        | and check if the files are **0kb**. If they are, **delete them** and run the function again.
+        | You may need to restart your previewer entirely for it to work!
 
-    Dependencies: TIVTC
+    Dependencies:
 
-    :param clip:                Input clip.
+    * `TIVTC <https://github.com/dubhater/vapoursynth-tivtc>`_
+
+    :param clip:                Clip to process.
     :param tfmIn:               File location for TFM's matches analysis.
                                 By default it will be written to ``.ivtc/{yourScriptName}_matches.txt``.
     :param tdecIn:              File location for TDecimate's metrics analysis.
@@ -158,7 +217,9 @@ def tivtc_vfr(clip: vs.VideoNode,
     :param tfm_args:            Additional arguments to pass to TFM.
     :param tdecimate_args:      Additional arguments to pass to TDecimate.
 
-    :return:                    IVTC'd VFR clip with external timecode/matches/metrics txt files
+    :return:                    IVTC'd VFR clip with external timecode/matches/metrics txt files.
+
+    :raises TypeError:          Invalid ``decimate`` argument is passed.
     """
     if int(decimate) not in (-1, 0, 1):
         raise TypeError("TIVTC_VFR: 'Invalid `decimate` argument. Must be True/False, their integer values, or -1!'")
@@ -179,6 +240,11 @@ def tivtc_vfr(clip: vs.VideoNode,
             p.parent.mkdir(parents=True)
 
     if not (tfm_f.exists() and tdec_f.exists()):
+        warnings.warn("tivtc_vfr: 'When calculating the matches and metrics for the first time, "
+                      "your previewer may error out! To fix this, simply refresh your previewer. "
+                      "If it still doesn't work, open the ``.ivtc`` directory and check if the files are 0kb. "
+                      "If they are, delete them and run the function again.'")
+
         tfm_analysis: Dict[str, Any] = {**tfm_args, 'output': str(tfm_f)}
         tdec_analysis: Dict[str, Any] = {'mode': 4, **tdecimate_args, 'output': str(tdec_f)}
 
@@ -215,8 +281,10 @@ def tivtc_vfr(clip: vs.VideoNode,
 def deblend(clip: vs.VideoNode, start: int = 0,
             rep: int | None = None, decimate: bool = True) -> vs.VideoNode:
     """
-    A simple function to fix deblending for interlaced video with an AABBA blending pattern,
-    where A is a regular frame and B is a blended frame.
+    Deblending function for blended AABBA patterns.
+
+    .. warning:
+        This function's base functionality and settings will be updated in a future version!
 
     Assuming there's a constant pattern of frames (labeled A, B, C, CD, and DA in this function),
     blending can be fixed by calculating the D frame by getting halves of CD and DA, and using that
@@ -230,15 +298,18 @@ def deblend(clip: vs.VideoNode, start: int = 0,
 
     Dependencies:
 
-    * RGSF (optional: 32 bit clip)
+    * `RGSF <https://github.com/IFeelBloated/RGSF>`_ (optional: 32 bit clip)
 
-    :param clip:        Input clip
-    :param start:       First frame of the pattern (Default: 0)
-    :param rep:         Repair mode for the deblended frames, no repair if None (Default: None)
-    :param decimate:    Decimate the video after deblending (Default: True)
+    :param clip:        Clip to process.
+    :param start:       First frame of the pattern (Default: 0).
+    :param rep:         Repair mode for the deblended frames, no repair if None (Default: None).
+    :param decimate:    Decimate the video after deblending (Default: True).
 
-    :return:            Deblended clip
+    :return:            Deblended clip.
     """
+    warnings.warn("deblend: 'This function's base functionality and settings "
+                  "will be updated in a future version!'", DeprecationWarning)
+
     blends_a = range(start + 2, clip.num_frames - 1, 5)
     blends_b = range(start + 3, clip.num_frames - 1, 5)
     expr_cd = ["z a 2 / - y x 2 / - +"]
@@ -250,8 +321,8 @@ def deblend(clip: vs.VideoNode, start: int = 0,
         else:
             if n in blends_a:
                 c, cd, da, a = clip[n - 1], clip[n], clip[n + 1], clip[n + 2]
-                debl = core.std.Expr([c, cd, da, a], expr_cd)
-                return pick_repair(clip)(debl, c, rep) if rep else debl
+                debl = core.akarin.Expr([c, cd, da, a], expr_cd)
+                return repair(debl, c, rep) if rep else debl
             return clip
 
     debl = core.std.FrameEval(clip, partial(deblend, clip=clip, rep=rep))
@@ -262,44 +333,53 @@ def deblend(clip: vs.VideoNode, start: int = 0,
 def decomb(clip: vs.VideoNode,
            tff: bool | int = True, mode: int = 1,
            decimate: bool = True, vinv: bool = False,
-           rep: int | None = None, show_mask: bool = False,
+           rep: int | Sequence[int] = 0,
+           show_mask: bool = False,
            tfm_args: Dict[str, Any] = {},
            tdec_args: Dict[str, Any] = {},
            vinv_args: Dict[str, Any] = {},
            qtgmc_args: Dict[str, Any] = {}) -> vs.VideoNode:
     """
-    A filter that performs relatively aggressive filtering to get rid of the combing on a interlaced/telecined source.
+    Perform relatively aggressive filtering to get rid of the combing on a interlaced/telecined source.
+
+    .. warning:
+        This function will be removed in a future version!
+
     Decimation can be disabled if the user wishes to decimate the clip themselves.
 
     Enabling vinverse will result in more aggressive decombing at the cost of potential detail loss.
     A reference clip can be passed with `ref`, which will be used by TFM to create the output frames.
 
-    Base function written by Midlifecrisis from the WEEB AUTISM server, modified by LightArrowsEXE.
+    Original function written by Midlifecrisis, modified by LightArrowsEXE.
 
     Dependencies:
 
-    * combmask
-    * havsfunc
-    * RGSF (optional: 32 bit clip)
+    * `combmask <https://drive.google.com/file/d/15E0Ua27AndT-0zSHHCC1iL5SZO09Ntbv/view?usp=sharing>`_
+    * `havsfunc <https://github.com/HomeOfVapourSynthEvolution/havsfunc>`_
+    * `RGSF <https://github.com/IFeelBloated/RGSF>`_ (optional: 32 bit clip)
 
-    :param clip:          Input clip
-    :param tff:           Top-Field-First
-    :param mode:          Sets the matching mode or strategy to use for TFM
-    :param decimate:      Decimate the video after deinterlacing (Default: True)
-    :param vinv:          Use vinverse to get rid of additional combing (Default: False)
-    :param rep:           Repair mode for repairing the decombed clip using the original clip (Default: None)
-    :param show_mask:     Return combmask
-    :param tfm_args:      Arguments to pass to TFM
-    :param vinv_args:     Arguments to pass to vinverse
-    :param qtgmc_args:    Arguments to pass to QTGMC
+    :param clip:                    Clip to process.
+    :param tff:                     Top-Field-First.
+    :param mode:                    Sets the matching mode or strategy to use for TFM.
+    :param decimate:                Decimate the video after deinterlacing (Default: True).
+    :param vinv:                    Use vinverse to get rid of additional combing (Default: False).
+    :param rep:                     Repair mode for repairing the decombed clip using the original clip (Default: None).
+    :param show_mask:               Return combmask.
+    :param tfm_args:                Arguments to pass to TFM.
+    :param vinv_args:               Arguments to pass to vinverse.
+    :param qtgmc_args:              Arguments to pass to QTGMC.
 
-    :return:              Decombed and optionally decimated clip
+    :return:                        Decombed and optionally decimated clip.
+
+    :raises ModuleNotFoundError:    Dependencies are missing.
     """
     try:
         from havsfunc import QTGMC
     except ModuleNotFoundError:
         raise ModuleNotFoundError("decomb: missing dependency `havsfunc`!")
 
+    warnings.warn("decomb: 'This function will be removed in a future version!'", FutureWarning)
+
     VFM_TFF = int(tff)
 
     tfm_kwargs: Dict[str, Any] = dict(order=VFM_TFF, mode=mode, chroma=True)
@@ -327,34 +407,34 @@ def decomb(clip: vs.VideoNode,
     decombed = core.std.FrameEval(clip, partial(_pp, clip=clip, pp=qtgmc_merged), clip)
 
     decombed = vinverse(decombed, **vinv_args) if vinv else decombed
-    decombed = pick_repair(clip)(decombed, clip, rep) if rep else decombed
+    decombed = repair(decombed, clip, rep)
     return core.tivtc.TDecimate(decombed, **tdec_args) if decimate else decombed
 
 
 def descale_fields(clip: vs.VideoNode, tff: bool = True,
                    width: int | None = None, height: int = 720,
-                   kernel: Kernel | str = Bicubic(b=0, c=0.5),
+                   kernel: Kernel | str = Bicubic(b=0, c=1/2),
                    src_top: float = 0.0) -> vs.VideoNode:
     """
-    Simple descaling wrapper for interwoven upscaled fields.
+    Descale interwoven upscaled fields, also known as a cross conversion.
+
     This function also sets a frameprop with the kernel that was used.
 
-    The kernel is set using an lvsfunc.Kernel object.
-    You can call these by doing for example ``kernel=lvf.kernels.Bilinear()``.
-    You can also set specific values manually. For example: ``kernel=lvf.kernels.Bicubic(b=0, c=1)``.
-    For more information, check the documentation on Kernels.
+    The kernel is set using an py:class:`vskernels.Kernel` object.
+    For more information, check the `vskernels documentation <https://vskernels.encode.moe/en/latest/>`_.
 
     ``src_top`` allows you to to shift the clip prior to descaling.
     This may be useful, as sometimes clips are shifted before or after the original upscaling.
 
-    :param clip:        Input clip
-    :param tff:         Top-field-first. `False` sets it to Bottom-Field-First
-    :param width:       Native width. Will be automatically determined if set to `None`
-    :param height:      Native height. Will be divided by two internally
-    :param kernel:      lvsfunc.Kernel object. This can also be a string (default: Catrom)
-    :param src_top:     Shifts the clip vertically during the descaling
+    :param clip:        Clip to process.
+    :param tff:         Top-field-first. `False` sets it to Bottom-Field-First.
+    :param width:       Native width. Will be automatically determined if set to `None`.
+    :param height:      Native height. Will be divided by two internally.
+    :param kernel:      py:class:`vskernels.Kernel` object used for the descaling.
+                        This can also be the string name of the kernel (Default: py:class:`vskernels.Catrom`).
+    :param src_top:     Shifts the clip vertically during the descaling.
 
-    :return:            Descaled GRAY clip
+    :return:            Descaled GRAY clip.
     """
     height_field = int(height/2)
     width = width or get_w(height, clip.width/clip.height)
@@ -371,30 +451,12 @@ def descale_fields(clip: vs.VideoNode, tff: bool = True,
     return weave_y.std.SetFieldBased(0)[::2]
 
 
-def bob(clip: vs.VideoNode, tff: bool | None = None) -> vs.VideoNode:
-    """
-    Very simple bobbing function. Shouldn't be used for regular filtering,
-    but as a very cheap bobber for other functions.
-
-    :param clip:    Input clip
-    :param tff:     Top-field-first. `False` sets it to Bottom-Field-First.
-                    If None, get the field order from the _FieldBased prop.
-
-    :return:        Bobbed clip
-    """
-    if get_prop(clip.get_frame(0), '_FieldBased', int) == 0 and tff is None:
-        raise vs.Error("bob: 'You must set `tff` for this clip!'")
-    elif isinstance(tff, (bool, int)):
-        clip = clip.std.SetFieldBased(int(tff) + 1)
-
-    return Catrom().scale(clip.std.SeparateFields(), clip.width, clip.height)
-
-
 def fix_telecined_fades(clip: vs.VideoNode, tff: bool | int | None = None,
                         thr: float = 2.2) -> vs.VideoNode:
     """
-    A filter that gives a mathematically perfect solution to fades made *after* telecining
-    (which made perfect IVTC impossible). This is an improved version of the Fix-Telecined-Fades plugin
+    Give a mathematically perfect solution to fades made *after* telecining (which made perfect IVTC impossible).
+
+    This is an improved version of the Fix-Telecined-Fades plugin
     that deals with overshoot/undershoot by adding a check.
 
     Make sure to run this *after* IVTC/deinterlacing!
@@ -410,15 +472,16 @@ def fix_telecined_fades(clip: vs.VideoNode, tff: bool | int | None = None,
     Taken from this gist and modified by LightArrowsEXE.
     <https://gist.github.com/blackpilling/bf22846bfaa870a57ad77925c3524eb1>
 
-    :param clip:        Input clip
-    :param tff:         Top-field-first. `False` sets it to Bottom-Field-First.
-                        If None, get the field order from the _FieldBased prop.
-    :param thr:         Threshold for when a field should be adjusted.
-                        Default is 2.2, which appears to be a safe value that doesn't
-                        cause it to do weird stuff with orphan fields.
+    :param clip:                    Clip to process.
+    :param tff:                     Top-field-first. `False` sets it to Bottom-Field-First.
+                                    If None, get the field order from the _FieldBased prop.
+    :param thr:                     Threshold for when a field should be adjusted.
+                                    Default is 2.2, which appears to be a safe value that doesn't
+                                    cause it to do weird stuff with orphan fields.
 
-    :return:            Clip with only fades fixed
+    :return:                        Clip with only fades fixed.
 
+    :raises TopFieldFirstError:     No automatic ``tff`` can be determined.
     """
     def _ftf(n: int, f: List[vs.VideoFrame]) -> vs.VideoNode:
         avg = (get_prop(f[0], 'PlaneStatsAverage', float),
@@ -426,8 +489,8 @@ def fix_telecined_fades(clip: vs.VideoNode, tff: bool | int | None = None,
 
         if avg[0] != avg[1]:
             mean = sum(avg) / 2
-            fixed = (sep[0].std.Expr(f"x {mean} {avg[0]} / dup {thr} <= swap 1 ? *"),
-                     sep[1].std.Expr(f"x {mean} {avg[1]} / *"))
+            fixed = (sep[0].akarin.Expr(f"x {mean} {avg[0]} / dup {thr} <= swap 1 ? *"),
+                     sep[1].akarin.Expr(f"x {mean} {avg[1]} / *"))
         else:
             fixed = sep  # type: ignore
 
@@ -435,7 +498,7 @@ def fix_telecined_fades(clip: vs.VideoNode, tff: bool | int | None = None,
 
     # I want to catch this before it reaches separateFields and give newer users a more useful error
     if get_prop(clip.get_frame(0), '_FieldBased', int) == 0 and tff is None:
-        raise vs.Error("fix_telecined_fades: 'You must set `tff` for this clip!'")
+        raise TopFieldFirstError("fix_telecined_fades")
     elif isinstance(tff, (bool, int)):
         clip = clip.std.SetFieldBased(int(tff) + 1)
 
@@ -456,12 +519,13 @@ def fix_telecined_fades(clip: vs.VideoNode, tff: bool | int | None = None,
     return ftf
 
 
-def ivtc_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
-                 interlaced: bool = True, dec: bool | None = None,
-                 bob_clip: vs.VideoNode | None = None, qtgmc_args: Dict[str, Any] = {}
-                 ) -> vs.VideoNode:
+def pulldown_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
+                     interlaced: bool = True, dec: bool | None = None,
+                     bob_clip: vs.VideoNode | None = None, qtgmc_args: Dict[str, Any] = {}
+                     ) -> vs.VideoNode:
     """
     Deinterlacing function for interlaced credits (60i/30p) on top of telecined video (24p).
+
     This is a combination of havsfunc's dec_txt60mc, ivtc_txt30mc, and ivtc_txt60mc functions.
     The credits are interpolated and decimated to match the output clip.
 
@@ -473,33 +537,38 @@ def ivtc_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
     apply this function, and `vsutil.insert_clip` the clip back into a properly IVTC'd clip.
     Alternatively, use `muvsfunc.VFRSplice` to splice the clip back in if you're dealing with a VFR clip.
 
-    :param clip:            Input clip. Framerate must be 30000/1001.
-    :param frame_ref:       First frame in the pattern. Expected pattern is ABBCD,
-                            except for when ``dec`` is enabled, in which case it's AABCD.
-    :param tff:             Top-field-first. `False` sets it to Bottom-Field-First.
-    :param interlaced:      60i credits. Set to false for 30p credits.
-    :param dec:             Decimate input clip as opposed to IVTC.
-                            Automatically enabled if certain fieldmatching props are found.
-                            Can be forcibly disabled by setting it to `False`.
-    :param bob_clip:        Custom bobbed clip. If `None`, uses a QTGMC clip.
-                            Framerate must be 60000/1001.
-    :param qtgmc_args:      Arguments to pass on to QTGMC.
-                            Accepts any parameter except for FPSDivisor and TFF.
+    :param clip:                    Clip to process. Framerate must be 30000/1001.
+    :param frame_ref:               First frame in the pattern. Expected pattern is ABBCD,
+                                    except for when ``dec`` is enabled, in which case it's AABCD.
+    :param tff:                     Top-field-first. `False` sets it to Bottom-Field-First.
+    :param interlaced:              60i credits. Set to false for 30p credits.
+    :param dec:                     Decimate input clip as opposed to IVTC.
+                                    Automatically enabled if certain fieldmatching props are found.
+                                    Can be forcibly disabled by setting it to `False`.
+    :param bob_clip:                Custom bobbed clip. If `None`, uses a QTGMC clip.
+                                    Framerate must be 60000/1001.
+    :param qtgmc_args:              Arguments to pass on to QTGMC.
+                                    Accepts any parameter except for FPSDivisor and TFF.
 
-    :return:                IVTC'd/decimated clip with deinterlaced credits
+    :return:                        IVTC'd/decimated clip with credits pulled down to 24p.
+
+    :raises ModuleNotFoundError:    Dependencies are missing.
+    :raises ValueError:             Clip does not have a framerate of 30000/1001 (29.97).
+    :raises TopFieldFirstError:     No automatic ``tff`` can be determined.
+    :raises InvalidFramerateError:  Bobbed clip does not have a framerate of 60000/1001 (59.94)
     """
     try:
         from havsfunc import QTGMC, DitherLumaRebuild
     except ModuleNotFoundError:
-        raise ModuleNotFoundError("ivtc_credits: missing dependency `havsfunc`!")
+        raise ModuleNotFoundError("pulldown_credits: missing dependency `havsfunc`!")
 
-    check_variable(clip, "ivtc_credits")
+    assert check_variable(clip, "pulldown_credits")
 
     if clip.fps != Fraction(30000, 1001):
-        raise ValueError("ivtc_credits: 'Your clip must have a framerate of 30000/1001!'")
+        raise ValueError("pulldown_credits: 'Your clip must have a framerate of 30000/1001!'")
 
     if get_prop(clip.get_frame(0), '_FieldBased', int) == 0 and tff is None:
-        raise vs.Error("ivtc_credits: 'You must set `tff` for this clip!'")
+        raise TopFieldFirstError("pulldown_credits")
     elif isinstance(tff, (bool, int)):
         clip = clip.std.SetFieldBased(int(tff) + 1)
 
@@ -511,8 +580,8 @@ def ivtc_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
         dec = any(x in clip.get_frame(0).props for x in {"VFMMatch", "TFMMatch"})
 
         if dec:
-            warnings.warn("ivtc_credits: 'Fieldmatched clip passed to function! "
-                          "dec is set to True. If you want to disable this, set `dec=False`!'")
+            warnings.warn("pulldown_credits: 'Fieldmatched clip passed to function! "
+                          "dec is set to `True`. If you want to disable this, set `dec=False`!'")
 
     # motion vector and other values
     field_ref = frame_ref * 2
@@ -533,7 +602,7 @@ def ivtc_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
     bobbed = bob_clip or QTGMC(clip, **qtgmc_kwargs)
 
     if bobbed.fps != Fraction(60000, 1001):
-        raise InvalidFramerateError("ivtc_credits", bobbed,
+        raise InvalidFramerateError("pulldown_credits", bobbed,
                                     "{func} 'Your bobbed clip *must* have a framerate of 60000/1001!'")
 
     if interlaced:  # 60i credits. Start of ABBCD
@@ -623,20 +692,22 @@ def ivtc_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
 def vinverse(clip: vs.VideoNode, sstr: float = 2.0,
              amount: int = 128, scale: float = 1.5) -> vs.VideoNode:
     """
-    A simple function to clean up residual combing after a deinterlacing pass.
-    This is Setsugen_no_ao's implementation, adopted into lvsfunc.
+    Clean up residual combing after a deinterlacing pass.
 
-    :param clip:    Input clip.
-    :param sstr:    Contrasharpening strength. Increase this if you find
-                    the decombing blurs the image a bit too much.
-    :param amount:  Maximum difference allowed between the original pixels and adjusted pixels.
-                    Scaled to input clip's depth. Set to 255 to effectively disable this.
-    :param scale:   Scale amount for vertical sharp * vertical blur.
+    This is Setsugen no ao's implementation, adopted into lvsfunc.
 
-    :return:        Clip with residual combing largely removed
+    :param clip:        Clip to process.
+    :param sstr:        Contrasharpening strength. Increase this if you find
+                        the decombing blurs the image a bit too much.
+    :param amount:      Maximum difference allowed between the original pixels and adjusted pixels.
+                        Scaled to input clip's depth. Set to 255 to effectively disable this.
+    :param scale:       Scale amount for vertical sharp * vertical blur.
+
+    :return:            Clip with residual combing largely removed.
+
+    :raises ValueError: ``amount`` is set above 255.
     """
-    check_variable_format(clip, "vinverse")
-    assert clip.format
+    assert check_variable_format(clip, "vinverse")
 
     if amount > 255:
         raise ValueError("vinverse: '`amount` may not be set higher than 255!'")
@@ -660,6 +731,30 @@ def vinverse(clip: vs.VideoNode, sstr: float = 2.0,
                             'merge@ > x a@ - merge@ ? ?')
 
 
+# helpers
+def _check_pattern(clip: vs.VideoNode, pattern: int = 0) -> bool:
+    """:py:func:`lvsfunc.deinterlace.check_patterns` rendering behaviour."""
+    clip = sivtc(clip, pattern)
+    clip = core.tdm.IsCombed(clip)
+
+    frames: List[int] = []
+
+    def _cb(n: int, f: vs.VideoFrame) -> None:
+        if get_prop(f, '_Combed', int):
+            frames.append(n)
+
+    # TODO: Tried being clever and just exiting if any combing was found, but async_render had other plans :)
+    clip_async_render(clip[::4], progress=f"Checking pattern {pattern}...", callback=_cb)
+
+    if len(frames) > 0:
+        print(f"check_patterns: 'Combing found with pattern {pattern}!'")
+        return False
+
+    print(f"check_patterns: 'Clean clip found with pattern {pattern}!'")
+    return True
+
+
 # Temporary aliases
 SIVTC = sivtc
 TIVTC_VFR = tivtc_vfr
+ivtc_credits = pulldown_credits
