@@ -1,19 +1,19 @@
 """
-    Very dumb "funny xdddd" functions.
+Dumb le funny xddd functions.
 
-    These are not intended to be used in any release whatsoever.
-    If you *do* use them, I sincerely hope it's for an April Fools encode.
+These are not intended to be used in any release whatsoever.
+If you *do* use them, I sincerely hope it's for an April Fools encode.
 
-    These functions are undocumented and will never be full features. Probably.
-    Don't tell louis this exists though, else I'm a dead man.
+These functions are undocumented and will never be full features. Probably.
+Don't tell louis this exists though, else I'm a dead man.
 """
 from __future__ import annotations
 
 from typing import List
 
 import vapoursynth as vs
+from vskernels import Point
 
-from .kernels import Point
 from .util import check_variable_resolution, force_mod
 
 core = vs.core
@@ -25,12 +25,13 @@ __all__: List[str] = [
 
 def minecraftify(clip: vs.VideoNode, div: float = 64.0, mod: int | None = None) -> vs.VideoNode:
     """
-    Function that transforms your clip into a Minecraft.
-    Idea from Meme-Maji's Kobayashi memery (love you varde).
+    Transform your clip into a Minecraft.
 
-    :param clip:    Input clip
-    :param div:     How much to divide the clip's resolution with
-    :param mod:     Force the downscaled clip to be MOD# compliant
+    Idea from Meme-Maji's Kobayashi memery (love you Vardë).
+
+    :param clip:     Clip to process.
+    :param div:     How much to divide the clip's resolution with.
+    :param mod:     Force the downscaled clip to be MOD# compliant.
 
     :return:        A Minecraft.
     """
