@@ -1,17 +1,19 @@
 from __future__ import annotations
 
+import math
 from abc import ABC, abstractmethod
 from functools import partial
-from typing import Any, Callable, Dict, List, Tuple
+from typing import Any, Callable, Dict, List, Sequence, Tuple
 
 import vapoursynth as vs
+from vsrgtools import removegrain
+from vsrgtools.util import normalise_planes
 from vsutil import Range as CRange
-from vsutil import depth, get_y, iterate, join, split
+from vsutil import depth, get_depth, get_y, iterate, join, split
 
 from . import util
 from .types import Position, Range, Shapes, Size
-from .util import (check_variable, check_variable_resolution, pick_removegrain,
-                   replace_ranges, scale_thresh)
+from .util import check_variable, check_variable_resolution, replace_ranges, scale_peak, scale_thresh
 
 core = vs.core
 
@@ -21,6 +23,7 @@ __all__: List[str] = [
     'DeferredMask',
     'detail_mask_neo',
     'detail_mask',
+    'fine_dehalo_mask',
     'halo_mask',
     'mt_xxpand_multi', 'minm', 'maxm',
     'range_mask',
@@ -30,18 +33,19 @@ __all__: List[str] = [
 def detail_mask(clip: vs.VideoNode, sigma: float | None = None,
                 rad: int = 3, brz_a: float = 0.025, brz_b: float = 0.045) -> vs.VideoNode:
     """
-    A wrapper for creating a detail mask to be used during denoising and/or debanding.
+    Create a detail mask to be used during denoising and/or debanding.
+
     The detail mask is created using debandshit's range mask,
     and is then merged with Prewitt to catch lines it may have missed.
 
     Dependencies:
 
-    * VapourSynth-Bilateral (optional: sigma)
-    * RGSF (optional: 32 bit clip)
+    * `VapourSynth-Bilateral <https://github.com/HomeOfVapourSynthEvolution/VapourSynth-Bilateral>`_ (optional: sigma)
+    * `RGSF <https://github.com/IFeelBloated/RGSF>`_ (optional: 32 bit clip)
 
-    :param clip:        Input clip
-    :param sigma:       Sigma for Bilateral for pre-blurring (Default: False)
-    :param rad:         The luma equivalent of gradfun3's "mask" parameter
+    :param clip:        Clip to process.
+    :param sigma:       Sigma for Bilateral for pre-blurring (Default: False).
+    :param rad:         The luma equivalent of gradfun3's "mask" parameter.
     :param brz_a:       Binarizing thresh for the detail mask.
                         Scaled to clip's depth if between 0 and 1 (inclusive),
                         else assumed to be in native range. (Default: 0.025)
@@ -49,7 +53,7 @@ def detail_mask(clip: vs.VideoNode, sigma: float | None = None,
                         Scaled to clip's depth if between 0 and 1 (inclusive),
                         else assumed to be in native range. (Default: 0.045)
 
-    :return:            Detail mask
+    :return:            Detail mask.
     """
     check_variable_resolution(clip, "detail_mask")
 
@@ -65,9 +69,9 @@ def detail_mask(clip: vs.VideoNode, sigma: float | None = None,
     mask_b = core.std.Prewitt(get_y(blur))
     mask_b = core.std.Binarize(mask_b, brz_b)
 
-    mask = core.std.Expr([mask_a, mask_b], 'x y max')
-    mask = util.pick_removegrain(mask)(mask, 22)
-    return util.pick_removegrain(mask)(mask, 11).std.Limiter()
+    mask = core.akarin.Expr([mask_a, mask_b], 'x y max')
+    mask = removegrain(mask, 22)
+    return removegrain(mask, 11).std.Limiter()
 
 
 def detail_mask_neo(clip: vs.VideoNode, sigma: float = 1.0,
@@ -75,12 +79,13 @@ def detail_mask_neo(clip: vs.VideoNode, sigma: float = 1.0,
                     blur_func: Callable[[vs.VideoNode, vs.VideoNode, float],
                                         vs.VideoNode] | None = None,
                     edgemask_func: Callable[[vs.VideoNode], vs.VideoNode] = core.std.Prewitt,
-                    rg_mode: int = 17) -> vs.VideoNode:
+                    rg_mode: int | Sequence[int] = 17) -> vs.VideoNode:
     """
-    A detail mask aimed at preserving as much detail as possible within darker areas,
-    even if it winds up being mostly noise.
+    Detail mask aimed at preserving as much detail as possible.
 
-    :param clip:            Input clip
+    This mask will catch a whole lot of stuff, including noise and grain.
+
+    :param clip:            Clip to process.
     :param sigma:           Sigma for the detail mask.
                             Higher means more detail and noise will be caught.
     :param detail_brz:      Binarizing for the detail mask.
@@ -92,13 +97,12 @@ def detail_mask_neo(clip: vs.VideoNode, sigma: float = 1.0,
     :param blur_func:       Blurring function used for the detail detection.
                             Must accept the following parameters: ``clip``, ``ref_clip``, ``sigma``.
                             Uses `bilateral.Bilateral` by default.
-    :param edgemask_func:   Edgemasking function used for the edge detection
-    :param rg_mode:         Removegrain mode performed on the final output
+    :param edgemask_func:   Edgemasking function used for the edge detection.
+    :param rg_mode:         Removegrain mode performed on the final output.
 
-    :return:                Detail mask
+    :return:                Detail mask.
     """
-    check_variable(clip, "detail_mask_neo")
-    assert clip.format
+    assert check_variable(clip, "detail_mask_neo")
 
     if not blur_func:
         blur_func = core.bilateral.Bilateral
@@ -110,7 +114,7 @@ def detail_mask_neo(clip: vs.VideoNode, sigma: float = 1.0,
     blur_pf = core.bilateral.Gaussian(clip_y, sigma=sigma / 4 * 3)
 
     blur_pref = blur_func(clip_y, blur_pf, sigma)
-    blur_pref_diff = core.std.Expr([blur_pref, clip_y], "x y -").std.Deflate()
+    blur_pref_diff = core.akarin.Expr([blur_pref, clip_y], "x y -").std.Deflate()
     blur_pref = iterate(blur_pref_diff, core.std.Inflate, 4)
 
     prew_mask = edgemask_func(clip_y).std.Deflate().std.Inflate()
@@ -120,8 +124,8 @@ def detail_mask_neo(clip: vs.VideoNode, sigma: float = 1.0,
     if lines_brz > 0:
         prew_mask = prew_mask.std.Binarize(lines_brz)
 
-    merged = core.std.Expr([blur_pref, prew_mask], "x y +")
-    rm_grain = pick_removegrain(merged)(merged, rg_mode)
+    merged = core.akarin.Expr([blur_pref, prew_mask], "x y +")
+    rm_grain = removegrain(merged, rg_mode)
 
     return depth(rm_grain, clip.format.bits_per_sample).std.Limiter()
 
@@ -132,28 +136,32 @@ def halo_mask(clip: vs.VideoNode, rad: int = 2,
               thlimi: float = 0.195, thlima: float = 0.392,
               edgemask: vs.VideoNode | None = None) -> vs.VideoNode:
     """
-    A halo mask to catch basic haloing, inspired by the mask from FineDehalo.
-    Most was copied from there, but some key adjustments were made to center it specifically around masking.
+    Halo mask to catch basic haloing.
+
+    Inspired by the mask from FineDehalo.
+
+    Most of it was copied from there, but some key adjustments have been made
+    to center it specifically around masking.
 
     rx and ry are now combined into rad and expects an integer.
     Float made sense for FineDehalo since it uses DeHalo_alpha for dehaloing,
     but the masks themselves use rounded rx/ry values, so there's no reason to bother with floats here.
 
-    All thresholds are float and will be scaled to ``clip``\\'s format.
+    All thresholds are float and will be scaled to ``clip``'s format.
     If thresholds are greater than 1, they will be asummed to be in 8-bit and scaled accordingly.
 
-    :param clip:            Input clip
-    :param rad:             Radius for the mask
-    :param brz:             Binarizing for shrinking mask (Default: 0.35)
-    :param thmi:            Minimum threshold for sharp edges; keep only the sharpest edges
-    :param thma:            Maximum threshold for sharp edges; keep only the sharpest edges
-    :param thlimi:          Minimum limiting threshold; includes more edges than previously, but ignores simple details
-    :param thlima:          Maximum limiting threshold; includes more edges than previously, but ignores simple details
-    :param edgemask:        Edgemask to use. If None, uses ``clip.std.Prewitt()`` (Default: None).
+    :param clip:            Clip to process.
+    :param rad:             Radius for the mask.
+    :param brz:             Binarizing for shrinking mask (Default: 0.35).
+    :param thmi:            Minimum threshold for sharp edges; keep only the sharpest edges.
+    :param thma:            Maximum threshold for sharp edges; keep only the sharpest edges.
+    :param thlimi:          Minimum limiting threshold; includes more edges than previously, but ignores simple details.
+    :param thlima:          Maximum limiting threshold; includes more edges than previously, but ignores simple details.
+    :param edgemask:        Edgemask to use. If None, uses :py:func:`vapoursynth.core.std.Prewitt` (Default: None).
 
-    :return:                Halo mask
+    :return:                Halo mask.
     """
-    check_variable(clip, "halo_mask")
+    assert check_variable(clip, "halo_mask")
 
     smax = scale_thresh(1.0, clip)
 
@@ -164,24 +172,105 @@ def halo_mask(clip: vs.VideoNode, rad: int = 2,
     edgemask = edgemask or get_y(clip).std.Prewitt()
 
     # Preserve just the strongest edges
-    strong = core.std.Expr(edgemask, expr=f"x {thmi} - {thlima-thlimi} / {smax} *")
+    strong = core.akarin.Expr(edgemask, expr=f"x {thmi} - {thlima-thlimi} / {smax} *")
     # Expand to pick up additional halos
     expand = iterate(strong, core.std.Maximum, rad)
 
     # Having too many intersecting lines will oversmooth the mask. We get rid of those here.
-    light = core.std.Expr(edgemask, expr=f"x {thlimi} - {thma-thmi} / {smax} *")
+    light = core.akarin.Expr(edgemask, expr=f"x {thlimi} - {thma-thmi} / {smax} *")
     shrink = iterate(light, core.std.Maximum, rad)
     shrink = core.std.Binarize(shrink, scale_thresh(brz, clip))
     shrink = iterate(shrink, core.std.Minimum, rad)
     shrink = iterate(shrink, partial(core.std.Convolution, matrix=matrix), 2)
 
     # Making sure the lines are actually excluded
-    excl = core.std.Expr([strong, shrink], expr="x y max")
+    excl = core.akarin.Expr([strong, shrink], expr="x y max")
     # Subtract and boosting to make sure we get the max pixel values for dehaloing
-    mask = core.std.Expr([expand, excl], expr="x y - 2 *")
+    mask = core.akarin.Expr([expand, excl], expr="x y - 2 *")
     # Additional blurring to amplify the mask
     mask = core.std.Convolution(mask, matrix)
-    return core.std.Expr(mask, expr="x 2 *").std.Limiter()
+    return core.akarin.Expr(mask, expr="x 2 *").std.Limiter()
+
+
+def fine_dehalo_mask(clip: vs.VideoNode,
+                     rx: float = 2.0, ry: float = 2.0,
+                     thmi: float = 80, thma: float = 128,
+                     thlimi: float = 50, thlima: float = 100,
+                     show_mask: bool | int = False) -> vs.VideoNode:
+    """
+    Create the mask for fine_dehalo.
+
+    This allows you to use the mask without having to jump through all the other code in fine_dehalo.
+
+    You can return the mask at different stages during the process with `show_mask`.
+
+    * 1 = Full mask (for backwards and fine_dehalo compatibility)
+
+    :param clip:            Clip to process.
+    :param rx:              Horizontal radius for halo removal. Must be greater than 1. Will be rounded up.
+    :param ry:              Vertical radius for halo removal. Must be greater than 1.  Will be rounded up.
+    :param thmi:            Minimum threshold for sharp edges. Keep only the sharpest edges (line edges).
+                            To see the effects of this setting take a look at the strong mask (show_mask=4).
+    :param thma:            Maximum threshold for sharp edges. Keep only the sharpest edges (line edges).
+                            To see the effects of this setting take a look at the strong mask (show_mask=4).
+    :param thlimi:          Minimum limiting threshold. Includes more edges than previously, but ignores simple details.
+    :param thlima:          Maximum limiting threshold. Includes more edges than previously, but ignores simple details.
+    :param show_mask:       Return mask clip at various stages in the operation. Valid options are 1–7.
+
+    :return:                Halo mask clip.
+
+    :raises ValueError:     ``rx`` or ``ry`` are smaller than 1.0.
+    :raises ValueError:     Invalid value for ``show_mask`` is passed.
+    """
+    assert check_variable(clip, "halo_mask")
+
+    clip_y = get_y(clip)
+
+    if not all(x >= 1 for x in (rx, ry)):
+        raise ValueError("halo_mask: '`rx` and `ry` must both be bigger than 1.0!'")
+
+    if show_mask is not False and not (0 < int(show_mask) <= 7):
+        raise ValueError("halo_mask: 'Valid values for `show_mask` are 1–7!'")
+
+    bits = get_depth(clip)
+    peak = (1 << bits) - 1
+    smax = scale_peak(255, peak)
+    thmi, thma, thlimi, thlima = map(partial(scale_peak, peak=peak), [thmi, thma, thlimi, thlima])
+    rx_i, ry_i = int(math.ceil(rx)), int(math.ceil(rx))
+
+    # Basic edge detection, thresholding will be applied later.
+    edges = clip_y.std.Prewitt()
+
+    # Keeps only the sharpest edges (line edges)
+    strong = core.akarin.Expr(edges, f"x {thmi} - {thma-thmi} / {smax} *")
+
+    # Extends them to include the potential halos
+    large: vs.VideoNode = maxm(strong, rx_i, ry_i)[-1]
+
+    # Includes more edges than previously, but ignores simple details
+    light = core.akarin.Expr(edges, f"x {thlimi} - {thlima-thlimi} / {smax} *")
+
+    # Growing the mask
+    shrink = maxm(light, sw=rx_i, sh=ry_i, mode=Shapes.ELLIPSE)[-1]
+    shrink = core.akarin.Expr(shrink, "x 2 *")
+    shrink = minm(shrink, sw=rx_i, sh=rx_i, mode=Shapes.ELLIPSE)[-1]
+    shrink = shrink.std.Convolution(matrix=[1] * 9).std.Convolution(matrix=[1] * 9)
+
+    shr_med = core.akarin.Expr([strong, shrink], expr="x y max")
+
+    mask = core.akarin.Expr([large, shr_med], expr="x y - 2 *")
+    mask = core.std.Convolution(mask, matrix=[1] * 9)
+    mask = core.akarin.Expr([mask], expr="x 2 *").std.Limiter()
+
+    match show_mask:
+        case 1: return mask
+        case 2: return shrink
+        case 3: return edges
+        case 4: return strong
+        case 5: return light
+        case 6: return large
+        case 7: return shr_med
+        case _: return mask
 
 
 def range_mask(clip: vs.VideoNode, rad: int = 2, radc: int = 0) -> vs.VideoNode:
@@ -198,29 +287,27 @@ def range_mask(clip: vs.VideoNode, rad: int = 2, radc: int = 0) -> vs.VideoNode:
 
     When radii are equal to 1, this filter becomes identical to mt_edge("min/max", 0, 255, 0, 255).
 
-    :param clip:    Input clip
-    :param rad:     Depth in pixels of the detail/edge masking
-    :param radc:    Chroma equivalent to ``rad``
+    :param clip:        Clip to process.
+    :param rad:         Depth in pixels of the detail/edge masking.
+    :param radc:        Chroma equivalent to ``rad``.
 
-    :return:        Range mask
+    :return:            Range mask.
     """
-    check_variable(clip, "range_mask")
+    assert check_variable(clip, "range_mask")
 
     if radc == 0:
         clip = get_y(clip)
 
-    assert clip.format
-
-    if clip.format.color_family == vs.GRAY:
+    if clip.format.color_family == vs.GRAY:  # type:ignore[union-attr]
         ma = _minmax(clip, rad, True)
         mi = _minmax(clip, rad, False)
-        mask = core.std.Expr([ma, mi], 'x y -')
+        mask = core.akarin.Expr([ma, mi], 'x y -')
     else:
         planes = split(clip)
         for i, rad_ in enumerate([rad, radc, radc]):
             ma = _minmax(planes[i], rad_, True)
             mi = _minmax(planes[i], rad_, False)
-            planes[i] = core.std.Expr([ma, mi], 'x y -')
+            planes[i] = core.akarin.Expr([ma, mi], 'x y -')
         mask = join(planes)
 
     return mask.std.Limiter()
@@ -228,8 +315,7 @@ def range_mask(clip: vs.VideoNode, rad: int = 2, radc: int = 0) -> vs.VideoNode:
 
 # Helper functions
 def _minmax(clip: vs.VideoNode, iterations: int, maximum: bool) -> vs.VideoNode:
-    check_variable(clip, "_minmax")
-    assert clip.format
+    assert check_variable(clip, "_minmax")
 
     func = core.std.Maximum if maximum else core.std.Minimum
 
@@ -242,16 +328,18 @@ def _minmax(clip: vs.VideoNode, iterations: int, maximum: bool) -> vs.VideoNode:
 
 class BoundingBox():
     """
-    A positional bounding box.
-    Basically kagefunc.squaremask but can be configured then deferred.
+    Positional bounding box.
+
+    Basically kagefunc.squaremask, but can be configured and then deferred.
 
     Uses Position + Size, like provided by GIMP's rectangle selection tool.
 
-    :param pos:  Offset of top-left corner of the bounding box from the top-left corner of the frame.
-                 Supports either a :py:class:`lvsfunc.types.Position` or a tuple that will be converted.
-    :param size: Offset of the bottom-right corner of the bounding box from the top-left corner of the bounding box.
-                 Supports either a :py:class:`lvsfunc.types.Size` or a tuple that will be converted.
+    :param pos:     Offset of top-left corner of the bounding box from the top-left corner of the frame.
+                    Supports either a :py:attr:`lvsfunc.types.Position` or a tuple that will be converted.
+    :param size:    Offset of the bottom-right corner of the bounding box from the top-left corner of the bounding box.
+                    Supports either a :py:attr:`lvsfunc.types.Size` or a tuple that will be converted.
     """
+
     pos: Position
     size: Size
 
@@ -261,14 +349,15 @@ class BoundingBox():
 
     def get_mask(self, ref: vs.VideoNode) -> vs.VideoNode:
         """
-        Get a mask representing the bounding box
+        Get a mask representing the bounding box.
 
-        :param ref: Reference clip for format, resolution, and length.
+        :param ref:             Reference clip for format, resolution, and length.
 
-        :return:    Square mask representing the bounding box.
+        :return:                Square mask representing the bounding box.
+
+        :raises ValueError:     Bound exceeds clip size.
         """
-        check_variable(ref, "get_mask")
-        assert ref.format
+        assert check_variable(ref, "get_mask")
 
         if self.pos.x + self.size.x > ref.width or self.pos.y + self.size.y > ref.height:
             raise ValueError("BoundingBox: Bounds exceed clip size!")
@@ -291,15 +380,17 @@ class DeferredMask(ABC):
     Provides an interface to use different preconfigured masking functions.
     Provides support for ranges, reference frames, and bounding.
 
-    :param range:    A single range or list of ranges to replace,
-                     compatible with :py:class:`lvsfunc.misc.replace_ranges`
-    :param bound:    A :py:class:`lvsfunc.mask.BoundingBox` or a tuple that will be converted.
-                     (Default: ``None``, no bounding)
-    :param blur:     Blur the bounding mask (Default: False)
-    :param refframe: A single frame number to use to generate the mask
-                     or a list of frame numbers with the same length as ``range``
+    :param range:       A single range or list of ranges to replace,
+                        compatible with :py:func:`lvsfunc.misc.replace_ranges`
+    :param bound:       A :py:class:`lvsfunc.mask.BoundingBox` or a tuple that will be converted
+                        (Default: ``None``, no bounding).
+    :param blur:        Blur the bounding mask (Default: False).
+    :param refframe:    A single frame number to use to generate the mask
+                        or a list of frame numbers with the same length as :py:func:`lvsfunc.types.Range`.
 
+    :raises ValueError: Reference frame and ranges mismatch.
     """
+
     ranges: List[Range]
     bound: BoundingBox | None
     refframes: List[int | None]
@@ -324,7 +415,7 @@ class DeferredMask(ABC):
         elif isinstance(refframes, int):
             self.refframes = [refframes] * len(self.ranges)
         else:
-            self.refframes = [x for x in refframes]  # wtf mypy
+            self.refframes = list(refframes)
 
         if len(self.refframes) > 0 and len(self.refframes) != len(self.ranges):
             raise ValueError("DeferredMask: 'Received reference frame and range list size mismatch!'")
@@ -333,15 +424,13 @@ class DeferredMask(ABC):
         """
         Get the bounded mask.
 
-        :param clip:  Source
-        :param ref:   Reference clip
+        :param clip:  Source.
+        :param ref:   Reference clip.
 
-        :return:      Bounded mask
+        :return:      Bounded mask.
         """
-        check_variable(clip, "get_mask")
-        check_variable(ref, "get_mask")
-        assert clip.format
-        assert ref.format
+        assert check_variable(clip, "get_mask")
+        assert check_variable(ref, "get_mask")
 
         if self.bound:
             bm = self.bound.get_mask(ref)
@@ -360,7 +449,7 @@ class DeferredMask(ABC):
                     rf = ref.num_frames - 1 + rf
                 mask = depth(self._mask(clip[rf], ref[rf]), clip.format.bits_per_sample,
                              range=CRange.FULL, range_in=CRange.FULL)
-                hm = replace_ranges(hm, core.std.Expr([hm, mask*len(hm)], expr="x y max"), range)
+                hm = replace_ranges(hm, core.akarin.Expr([hm, mask*len(hm)], expr="x y max"), range)
 
         hm = hm.std.Limiter()
 
@@ -379,12 +468,13 @@ def mt_xxpand_multi(clip: vs.VideoNode,
                     planes: List[int] = [0, 1, 2],
                     **m_params: Any) -> List[vs.VideoNode]:
     """
-    Mask expanding/inpanding function written by Zastin.
+    Mask expanding/inpanding function written by `Zastin <https://github.com/kgrabs>`_.
 
     Performs multiple Minimums/Maximums.
     """
-    check_variable(clip, "mt_xxpand_multi")
-    assert clip.format
+    assert check_variable(clip, "mt_xxpand_multi")
+
+    planes = normalise_planes(clip, planes)
 
     params: Dict[str, Any] = dict(planes=planes)
     params |= m_params
@@ -393,21 +483,20 @@ def mt_xxpand_multi(clip: vs.VideoNode,
 
     match mode:
         case Shapes.ELLIPSE: coordinates = [[1] * 8, [0, 1, 0, 1, 1, 0, 1, 0],
-                                                     [0, 1, 0, 1, 1, 0, 1, 0]]
+                                            [0, 1, 0, 1, 1, 0, 1, 0]]
         case Shapes.LOSANGE: coordinates = [[0, 1, 0, 1, 1, 0, 1, 0]] * 3
         case _: coordinates = [[1] * 8] * 3
 
-    clips = [clip]
+    clips: List[vs.VideoNode] = [clip]
     end = int(min(sw, sh)) + start
 
     for x in range(start, end):
         clips += [m__imum(clips[-1], coordinates=coordinates[x % 3], **params)]
-
     for x in range(end, int(end + sw - sh)):
         clips += [m__imum(clips[-1], coordinates=[0, 0, 0, 1, 1, 0, 0, 0], **params)]
-
     for x in range(end, int(end + sh - sw)):
-        clips += [m__imum(clips[-1], coordinates=[0, 1, 0, 0, 0, 0, 1, 0], **params).std.Limiter()]
+        clips += [m__imum(clips[-1], coordinates=[0, 1, 0, 0, 0, 0, 1, 0], **params)
+                  .std.Limiter()]
 
     return clips
 
