@@ -21,9 +21,11 @@ __all__: List[str] = [
 
 def chroma_reconstruct(clip: vs.VideoNode, radius: int = 2, i444: bool = False) -> vs.VideoNode:
     """
-    A function to demangle messed-up chroma, like for example chroma
-    that was downscaled using Nearest Neighbour, or the chroma found on DVDs.
+    Chroma reconstruction filter using regress.
+
     This function should be used with care, and not blindly applied to anything.
+    Ideally you should see how the function works,
+    and then mangle the luma of your source to match how your chroma was mangled.
 
     This function can also return a 4:4:4 clip. This is not recommended
     except for very specific cases, like for example where you're
@@ -32,16 +34,15 @@ def chroma_reconstruct(clip: vs.VideoNode, radius: int = 2, i444: bool = False)
 
     Original function by shane, modified by Ichunjo and LightArrowsEXE.
 
-    Aliases for this function are `lvsfunc.demangle` and `lvsfunc.crecon`.
+    Aliases for this function are ``lvsfunc.demangle`` and ``lvsfunc.crecon``.
 
-    :param clip:    Input clip
-    :param radius:  Boxblur radius
-    :param i444:    Return a 4:4:4 clip
+    :param clip:     Clip to process.
+    :param radius:  Boxblur radius.
+    :param i444:    Return a 4:4:4 clip.
 
-    :return:        Clip with demangled chroma in either 4:2:0 or 4:4:4
+    :return:        Clip with demangled chroma in either 4:2:0 or 4:4:4.
     """
-    check_variable(clip, "chroma_reconstruct")
-    assert clip.format
+    assert check_variable(clip, "chroma_reconstruct")
 
     def dmgl(clip: vs.VideoNode) -> vs.VideoNode:
         return core.resize.Bicubic(clip, w, h, src_left=0.25)
@@ -69,16 +70,19 @@ def chroma_reconstruct(clip: vs.VideoNode, radius: int = 2, i444: bool = False)
 
 def regress(x: vs.VideoNode, *ys: vs.VideoNode, radius: int = 2, eps: float = 1e-7) -> List[RegressClips]:
     """
+    Regress a clip using mangled luma and chroma.
+
     Fit a line for every neighborhood of values of a given size in a clip
     with corresponding neighborhoods in one or more other clips.
 
     For more info see `this Wikipedia article <https://en.wikipedia.org/wiki/Simple_linear_regression>`_.
+
+    :raises ValueError:     ``radius`` is lesser than 0.
     """
-
     if radius <= 0:
-        raise ValueError("Regress: 'radius must be greater than zero!'")
+        raise ValueError("Regress: 'radius must be greater than 0!'")
 
-    Expr = core.std.Expr
+    Expr = core.akarin.Expr
     E = partial(vs.core.std.BoxBlur, hradius=radius, vradius=radius)
 
     def mul(*c: vs.VideoNode) -> vs.VideoNode:
@@ -109,7 +113,7 @@ def regress(x: vs.VideoNode, *ys: vs.VideoNode, radius: int = 2, eps: float = 1e
 
 def reconstruct_multi(c: vs.VideoNode, r: RegressClips, radius: int = 2) -> vs.VideoNode:
     """
-    Tries to reconstruct regressed clips using a base video clip.
+    Reconstruct regressed clips using a base video clip.
 
     :param c:       Original clip.
     :param r:       Regressed clips.
@@ -117,12 +121,12 @@ def reconstruct_multi(c: vs.VideoNode, r: RegressClips, radius: int = 2) -> vs.V
 
     :returns:       Fixed regressed clips.
     """
-    check_variable(c, "reconstruct_multi")
+    assert check_variable(c, "reconstruct_multi")
 
-    weights = core.std.Expr(r.correlation, 'x 0.5 - 0.5 / 0 max')
-    slope_pm = core.std.Expr((r.slope, weights), 'x y *')
+    weights = core.akarin.Expr(r.correlation, 'x 0.5 - 0.5 / 0 max')
+    slope_pm = core.akarin.Expr((r.slope, weights), 'x y *')
     slope_pm_sum = _mean(slope_pm, radius)
-    recons = core.std.Expr((c, slope_pm_sum), 'x y *')
+    recons = core.akarin.Expr((c, slope_pm_sum), 'x y *')
     return recons
 
 
