@@ -5,16 +5,13 @@ from functools import partial
 from typing import Any, Callable, Dict, List, cast
 
 import vapoursynth as vs
+from vskernels import Bicubic, BicubicSharp, Catrom, Kernel, Spline36, get_kernel, get_matrix
 from vsutil import depth, get_depth, get_w, get_y, iterate, join, plane
 
 from .exceptions import CompareSameKernelError
-from .kernels import (Bicubic, BicubicSharp, Catrom, Kernel, Spline36,
-                      get_kernel)
-from .types import (CURVES, CreditMask, CustomScaler, Resolution, ScaleAttempt,
-                    VSFunction)
-from .util import (check_variable, check_variable_format,
-                   check_variable_resolution, get_coefs, get_matrix,
-                   get_matrix_curve, get_prop, quick_resample, scale_thresh)
+from .types import CURVES, CreditMask, CustomScaler, Resolution, ScaleAttempt, VSFunction
+from .util import (check_variable, check_variable_format, check_variable_resolution, get_coefs, get_matrix_curve,
+                   get_prop, quick_resample, scale_thresh)
 
 try:
     from cytoolz import functoolz
@@ -53,7 +50,7 @@ def _perform_descale(resolution: Resolution, clip: vs.VideoNode,
     descaled = kernel.descale(clip, resolution.width, resolution.height) \
         .std.SetFrameProp('descaleResolution', intval=resolution.height)
     rescaled = kernel.scale(descaled, clip.width, clip.height)
-    diff = core.std.Expr([rescaled, clip], 'x y - abs').std.PlaneStats()
+    diff = core.akarin.Expr([rescaled, clip], 'x y - abs').std.PlaneStats()
     return ScaleAttempt(descaled, rescaled, resolution, diff)
 
 
@@ -83,24 +80,25 @@ def reupscale(clip: vs.VideoNode,
               kernel: Kernel | str = Bicubic(b=0, c=1/2),
               **kwargs: Any) -> vs.VideoNode:
     """
-    A quick 'n easy wrapper used to re-upscale a clip descaled with descale using znedi3.
+    Quick 'n easy wrapper to re-upscale a clip descaled with descale using znedi3.
 
     Function is curried to allow parameter tuning when passing to :py:func:`lvsfunc.scale.descale`
 
-    Stolen from Varde with some adjustments made.
+    Original function written by `Vardë <https://github.com/Ichunjo>`_, modified by LightArrowsEXE.
 
     Dependencies:
 
-    * znedi3
+    * `znedi3 <https://github.com/sekrit-twc/znedi3>`_
 
-    :param clip:         Input clip
-    :param width:        Upscale width. If None, determine from `height` assuming 16:9 aspect ratio (Default: None)
-    :param height:       Upscale height (Default: 1080)
-    :param kernel:       Kernel used to downscale the doubled clip (see :py:class:`lvsfunc.kernels.Kernel`,
-                         Default: kernels.Bicubic(b=0, c=1/2))
-    :param kwargs:       Arguments passed to znedi3 (Default: nsize=4, nns=4, qual=2, pscrn=2)
+    :param clip:        Clip to process.
+    :param width:       Upscale width. If None, determine from `height` assuming 16:9 aspect ratio (Default: None).
+    :param height:      Upscale height (Default: 1080).
+    :param kernel:      py:class:`vskernels.Kernel` object used for downscaling the super-sampled clip.
+                        This can also be the string name of the kernel
+                        (Default: py:class:`vskernels.Bicubic(b=0, c=1/2)`).
+    :param kwargs:      Arguments passed to znedi3 (Default: nsize=4, nns=4, qual=2, pscrn=2).
 
-    :return:             Reupscaled clip
+    :return:            Reupscaled clip.
     """
     width = width or get_w(height)
 
@@ -125,21 +123,20 @@ def reupscale(clip: vs.VideoNode,
 def descale_detail_mask(clip: vs.VideoNode, rescaled_clip: vs.VideoNode,
                         threshold: float = 0.05) -> vs.VideoNode:
     """
-    Generate a detail mask given a clip and a clip rescaled with the same
-    kernel.
+    Generate a detail mask given a clip and a clip rescaled with the same kernel.
 
     Function is curried to allow parameter tuning when passing to :py:func:`lvsfunc.scale.descale`
 
-    :param clip:           Original clip
-    :param rescaled_clip:  Clip downscaled and reupscaled using the same kernel
-    :param threshold:      Binarization threshold for mask (Default: 0.05)
+    :param clip:           Original clip.
+    :param rescaled_clip:  Clip downscaled and reupscaled using the same kernel.
+    :param threshold:      Binarization threshold for mask (Default: 0.05).
 
-    :return:               Mask of lost detail
+    :return:               Mask of lost detail.
     """
     check_variable_resolution(clip, "descale_detail_mask")
     check_variable_resolution(rescaled_clip, "descale_detail_mask")
 
-    mask = core.std.Expr([get_y(clip), get_y(rescaled_clip)], 'x y - abs') \
+    mask = core.akarin.Expr([get_y(clip), get_y(rescaled_clip)], 'x y - abs') \
         .std.Binarize(threshold)
     mask = iterate(mask, core.std.Maximum, 4)
     return iterate(mask, core.std.Inflate, 2).std.Limiter()
@@ -156,7 +153,8 @@ def descale(clip: vs.VideoNode,
             = descale_detail_mask, src_left: float = 0.0, src_top: float = 0.0,
             show_mask: bool = False) -> vs.VideoNode:
     """
-    A unified descaling function.
+    Unified descaling function.
+
     Includes support for handling fractional resolutions (experimental),
     multiple resolutions, detail masking, and conditional scaling.
 
@@ -170,36 +168,40 @@ def descale(clip: vs.VideoNode,
 
     Dependencies:
 
-    * vapoursynth-descale
-    * znedi3
+    * `VapourSynth-descale <https://github.com/Irrational-Encoding-Wizardry/VapourSynth-descale>`_
+    * `znedi3 <https://github.com/sekrit-twc/znedi3>`_
 
-    :param clip:                    Clip to descale
-    :param upscaler:                Callable function with signature upscaler(clip, width, height)
-                                    -> vs.VideoNode to be used for reupscaling.
-                                    Must be capable of handling variable res clips
-                                    for multiple heights and conditional scaling.
-                                    If a single height is given and upscaler is None,
-                                    a constant resolution GRAY clip will be returned instead.
-                                    Note that if upscaler is None, no upscaling will be performed
-                                    and neither detail masking nor proper fractional descaling can be performed.
-                                    (Default: :py:func:`lvsfunc.scale.reupscale`)
-    :param width:                   Width(s) to descale to (if None, auto-calculated)
-    :param height:                  Height(s) to descale to. List indicates multiple resolutions,
-                                    the function will determine the best. (Default: 720)
-    :param kernel:                  Kernel used to descale the clip. This can also be a string.
-                                    (see :py:class:`lvsfunc.kernels.Kernel`, (Default: kernels.Bicubic(b=0, c=1/2))
-    :param threshold:               Error threshold for conditional descaling (Default: 0.0, always descale)
-    :param mask:                    Function or mask clip used to mask detail. If ``None``, no masking.
-                                    Function must accept a clip and a reupscaled clip and return a mask.
-                                    (Default: :py:func:`lvsfunc.scale.descale_detail_mask`)
-    :param src_left:                Horizontal shifting for fractional resolutions (Default: 0.0)
-    :param src_top:                 Vertical shifting for fractional resolutions (Default: 0.0)
-    :param show_mask:               Return detail mask
+    :param clip:        Clip to descale.
+    :param upscaler:        Callable function with signature upscaler(clip, width, height)
+                            -> vs.VideoNode to be used for reupscaling.
+                            Must be capable of handling variable res clips
+                            for multiple heights and conditional scaling.
 
-    :return:                       Descaled and re-upscaled clip with float bitdepth
+                            If a single height is given and upscaler is None,
+                            a constant resolution GRAY clip will be returned instead.
+
+                            Note that if upscaler is None, no upscaling will be performed
+                            and neither detail masking nor proper fractional descaling can be performed.
+                            (Default: :py:func:`lvsfunc.scale.reupscale`)
+    :param width:           Width(s) to descale to (if None, auto-calculated).
+    :param height:          Height(s) to descale to. List indicates multiple resolutions,
+                            the function will determine the best. (Default: 720)
+    :param kernel:          py:class:`vskernels.Kernel` object used for the descaling.
+                            This can also be the string name of the kernel (Default: py:class:`vskernels.Catrom`).
+    :param threshold:       Error threshold for conditional descaling (Default: 0.0, always descale).
+    :param mask:            Function or mask clip used to mask detail. If ``None``, no masking.
+                            Function must accept a clip and a reupscaled clip and return a mask.
+                            (Default: :py:func:`lvsfunc.scale.descale_detail_mask`)
+    :param src_left:        Horizontal shifting for fractional resolutions (Default: 0.0).
+    :param src_top:         Vertical shifting for fractional resolutions (Default: 0.0).
+    :param show_mask:       Return detail mask.
+
+    :return:                Descaled and re-upscaled clip with float bitdepth.
+
+    :raises ValueError:     Asymmetric number of heights and widths are specified.
+    :raises RuntimeError:   Variable clip gets returned.
     """
-    check_variable(clip, "descale")
-    assert clip.format
+    assert check_variable(clip, "descale")
 
     if isinstance(kernel, str):
         kernel = get_kernel(kernel)()
@@ -222,7 +224,6 @@ def descale(clip: vs.VideoNode,
     resolutions = [Resolution(*r) for r in zip(width, height)]
 
     clip = depth(clip, 32)
-    assert clip.format is not None  # clip was modified by depth, but that won't make it variable
     clip_y = get_y(clip) \
         .std.SetFrameProp('descaleResolution', intval=clip.height)
 
@@ -267,7 +268,7 @@ def descale(clip: vs.VideoNode,
         clip_y = clip_y.resize.Point(format=upscaled.format.id)
         rescaled = kernel.scale(descaled, clip.width, clip.height,
                                 (src_left, src_top))
-        rescaled = rescaled.resize.Point(format=clip.format.id)
+        rescaled = rescaled.resize.Point(format=clip.format.id)  # type:ignore[union-attr]
 
         dmask = mask if isinstance(mask, vs.VideoNode) else mask(clip_y, rescaled)
 
@@ -282,18 +283,19 @@ def descale(clip: vs.VideoNode,
 
     upscaled = depth(upscaled, get_depth(clip))
 
-    if clip.format.num_planes == 1 or upscaler is None:
+    if clip.format.num_planes == 1 or upscaler is None:  # type:ignore[union-attr]
         return upscaled
     return join([upscaled, plane(clip, 1), plane(clip, 2)])
 
 
 def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int = 720,
                     smooth: float | VSFunction = ((3 ** 2 - 1) / 12) ** 0.5,
-                    kernel: Kernel | str = Catrom(), gamma: bool = False,
+                    kernel: Kernel | str = Bicubic(b=0, c=1/2), gamma: bool = False,
                     curve: CURVES | None = None,
                     sigmoid: bool = False, epsilon: float = 1e-6) -> vs.VideoNode:
     """
-    muvsfunc.SSIM_downsample rewrite taken from a Vardë gist.
+    SSIM_downsample rewrite taken from a Vardë gist.
+
     Unlike muvsfunc's implementation, this function also works in float and does not use nnedi3_resample.
     Most of the documentation is taken from muvsfunc.
 
@@ -308,9 +310,9 @@ def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int =
 
     `Original gist <https://gist.github.com/Ichunjo/16ab1f893588aafcb096c1f35a0cfb15>`_
 
-    :param clip:        Input clip.
+    :param clip:        Clip to process.
     :param width:       Output width. If None, autocalculates using height.
-    :param height:      Output height (default: 720).
+    :param height:      Output height (Default: 720).
     :param smooth:      Image smoothening method.
                         If you pass an int, it specifies the "radius" of the internally-used boxfilter,
                         i.e. the window has a size of (2*smooth+1)x(2*smooth+1).
@@ -318,6 +320,9 @@ def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int =
                         i.e. the standard deviation of gaussian blur.
                         If you pass a function, it acts as a general smoother.
                         Default uses a gaussian blur.
+    :param kernel:      py:class:`vskernels.Kernel` object used for certain scaling operations.
+                        This can also be the string name of the kernel
+                        (Default: py:class:`vskernels.Bicubic(0, 0.5)`).
     :param curve:       Gamma mapping. Will auto-determine based on the input props or resolution.
                         Can be forced with for example `curve=vs.TransferCharacteristics.TRANSFER_BT709`.
     :param gamma:       Perform a gamma conversion prior to scaling and after scaling.
@@ -331,7 +336,7 @@ def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int =
 
     :return:            Downsampled clip.
     """
-    check_variable(clip, "ssim_downsample")
+    assert check_variable(clip, "ssim_downsample")
 
     if isinstance(kernel, str):
         kernel = get_kernel(kernel)()
@@ -349,27 +354,22 @@ def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int =
     clip = depth(clip, 32)
 
     if gamma:
-        if curve is None:
-            try:
-                curve = get_matrix_curve(get_prop(clip.get_frame(0), "_Matrix", int))
-            except ValueError:
-                curve = get_matrix_curve(get_matrix(clip, return_matrix=False))
-
+        curve = curve or get_matrix_curve(get_matrix(clip))
         clip = gamma2linear(clip, curve, sigmoid=sigmoid, epsilon=epsilon)
 
     l1 = kernel.scale(clip, width, height)
-    l2 = kernel.scale(clip.std.Expr('x dup *'), width, height)
+    l2 = kernel.scale(clip.akarin.Expr('x dup *'), width, height)
 
     m = filter_func(l1)
 
-    sl_plus_m_square = filter_func(l1.std.Expr('x dup *'))
+    sl_plus_m_square = filter_func(l1.akarin.Expr('x dup *'))
     sh_plus_m_square = filter_func(l2)
-    m_square = m.std.Expr('x dup *')
-    r = core.std.Expr([sl_plus_m_square, sh_plus_m_square, m_square], f'x z - {epsilon} < 0 y z - x z - / sqrt ?')
-    t = filter_func(core.std.Expr([r, m], 'x y *'))
+    m_square = m.akarin.Expr('x dup *')
+    r = core.akarin.Expr([sl_plus_m_square, sh_plus_m_square, m_square], f'x z - {epsilon} < 0 y z - x z - / sqrt ?')
+    t = filter_func(core.akarin.Expr([r, m], 'x y *'))
     m = filter_func(m)
     r = filter_func(r)
-    d = core.std.Expr([m, r, l1, t], 'x y z * + a -')
+    d = core.akarin.Expr([m, r, l1, t], 'x y z * + a -')
 
     if gamma:
         curve = cast(CURVES, curve)
@@ -380,9 +380,8 @@ def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int =
 def gamma2linear(clip: vs.VideoNode, curve: CURVES, gcor: float = 1.0,
                  sigmoid: bool = False, thr: float = 0.5, cont: float = 6.5,
                  epsilon: float = 1e-6) -> vs.VideoNode:
-    """Converts from gamma to linear."""
-    check_variable_format(clip, "gamma2linear")
-    assert clip.format
+    """Convert gamma to linear."""
+    assert check_variable_format(clip, "gamma2linear")
 
     if get_depth(clip) != 32 and clip.format.sample_type != vs.FLOAT:
         raise ValueError("gamma2linear: 'Your clip must be 32bit float!'")
@@ -397,15 +396,14 @@ def gamma2linear(clip: vs.VideoNode, curve: CURVES, gcor: float = 1.0,
 
     expr = f'{expr} 0.0 max 1.0 min'
 
-    return core.std.Expr(clip, expr).std.SetFrameProps(_Transfer=8)
+    return core.akarin.Expr(clip, expr).std.SetFrameProps(_Transfer=8)
 
 
 def linear2gamma(clip: vs.VideoNode, curve: CURVES, gcor: float = 1.0,
                  sigmoid: bool = False, thr: float = 0.5, cont: float = 6.5,
                  ) -> vs.VideoNode:
-    """Converts from linear to gamma."""
-    check_variable_format(clip, "linear2gamma")
-    assert clip.format
+    """Convert linear to gamma."""
+    assert check_variable_format(clip, "linear2gamma")
 
     if get_depth(clip) != 32 and clip.format.sample_type != vs.FLOAT:
         raise ValueError("linear2gamma: 'Your clip must be 32bit float!'")
@@ -422,24 +420,28 @@ def linear2gamma(clip: vs.VideoNode, curve: CURVES, gcor: float = 1.0,
     expr = f'{expr} {c.k0} {c.phi} / <= {expr} {c.phi} * {expr} 1 {c.gamma} / pow {c.alpha} 1 + * {c.alpha} - ?'
     expr = f'{expr} 0.0 max 1.0 min'
 
-    return core.std.Expr(clip, expr).std.SetFrameProps(_Transfer=curve)
+    return core.akarin.Expr(clip, expr).std.SetFrameProps(_Transfer=curve)
 
 
 def comparative_descale(clip: vs.VideoNode, width: int | None = None, height: int = 720,
                         kernel: Kernel | str | None = None, thr: float = 5e-8) -> vs.VideoNode:
     """
-    Easy wrapper to descale to SharpBicubic and an additional kernel,
-    compare them, and then pick one or the other.
+    Descale to SharpBicubic and an additional kernel, compare them, then pick one or the other.
 
     The output clip has props that can be used to frameeval specific kernels by the user.
 
-    :param clip:        Input clip
-    :param width:       Width to descale to (if None, auto-calculated)
-    :param height:      Descale height
-    :param kernel:      Kernel to compare BicubicSharp to. This can also be a string (Default: Spline36 if None).
-    :param thr:         Threshold for which kernel to pick
+    :param clip:                        Clip to process.
+    :param width:                       Width to descale to (if None, auto-calculated).
+    :param height:                      Descale height.
+    :param kernel:                      py:class:`vskernels.Kernel` object to compare
+                                        py:class:`vskernels.BicubicSharp` to.
+                                        This can also be the string name of the kernel
+                                        (Default: py:class:`vskernels.Spline36` if ``kernel=None``).
+    :param thr:                         Threshold for which kernel to pick.
 
-    :return:            Descaled clip
+    :return:                            Descaled clip.
+
+    :raises CompareSameKernelError:     py:class:`vskernels.BicubicSharp` gets passed to ``kernel``.
     """
     def _compare(n: int, f: vs.VideoFrame, sharp: vs.VideoNode, other: vs.VideoNode) -> vs.VideoNode:
         sharp_diff = get_prop(f[0], 'PlaneStatsDiff', float)  # type:ignore[arg-type]
@@ -484,15 +486,19 @@ def comparative_descale(clip: vs.VideoNode, width: int | None = None, height: in
 def comparative_restore(clip: vs.VideoNode, width: int | None = None, height: int = 720,
                         kernel: Kernel | str | None = None) -> vs.VideoNode:
     """
-    Companion function to go with comparative_descale
-    to reupscale the clip for descale detail masking.
+    Companion function for comparative_descale to reupscale the clip for descale detail masking.
 
-    :param clip:        Input clip
-    :param width:       Width to upscale to (if None, auto-calculated)
-    :param height:      Upscale height
-    :param kernel:      Kernel to compare BicubicSharp to (Default: Spline36 if None)
+    :param clip:                        Clip to process.
+    :param width:                       Width to upscale to (if None, auto-calculated).
+    :param height:                      Upscale height.
+    :param kernel:                      py:class:`vskernels.Kernel` object to compare
+                                        py:class:`vskernels.BicubicSharp` to.
+                                        This can also be the string name of the kernel
+                                        (Default: py:class:`vskernels.Spline36` if ``kernel=None``).
 
-    :return:            Reupscaled clip
+    :return:                            Reupscaled clip.
+
+    :raises CompareSameKernelError:     py:class:`vskernels.BicubicSharp` gets passed to ``kernel``.
     """
     check_variable_resolution(clip, "comparative_restore")
 
@@ -525,40 +531,44 @@ def mixed_rescale(clip: vs.VideoNode, width: None | int = None, height: int = 72
                   mix_strength: float = 0.25, show_mask: bool | int = False,
                   nnedi3_args: Dict[str, Any] = {}, eedi3_args: Dict[str, Any] = {}) -> vs.VideoNode:
     """
-    InsaneAA rewrite to be a much saner and easier to read function.
+    Rewrite of InsaneAA to make it easier to use and maintain.
+
     Descales and downscales the given clip and merges them together with a set strength.
 
     This can be useful for dealing with a source that you can't accurately descale,
     but you still want to force it. Not recommended to use it on everything, however.
 
     A string can be passed instead of a Kernel object if you want to use that.
-    This gives you access to every kernel object in :py:func:`lvsfunc.kernels`.
+    This gives you access to every kernel object in :py:mod:`vskernels`.
     For more information on what every kernel does, please refer to their documentation.
 
     This is still a work in progress at the time of writing. Please use with care.
 
-    :param clip:            Input clip.
-    :param width:           Upscale width. If None, determine from `height` (default: None).
+    :param clip:            Clip to process.
+    :param width:           Upscale width. If None, determine from `height` (Default: None).
     :param height:          Upscale height (Default: 720).
-    :param kernel:          Kernel used to descale the clip. This can also be a string.
-                            (see :py:class:`lvsfunc.kernels.Kernel`, Default: kernels.Bicubic(b=0, c=1/2) (Catrom)).
-    :param downscaler:      Kernel or custom scaler used to downscale the clip. This can also be a string.
-                            (see :py:class:`lvsfunc.kernels.Kernel`, Default: ``ssim_downsample``).
+    :param kernel:          py:class:`vskernels.Kernel` object used for the descaling.
+                            This can also be the string name of the kernel
+                            (Default: py:class:`vskernels.Bicubic(b=0, c=1/2)`).
+    :param downscaler:      Kernel or custom scaler used to downscale the clip.
+                            This can also be the string name of the kernel
+                            (Default: py:func:`lvsfunc.scale.ssim_downsample`).
     :param credit_mask:     Function or mask clip used to mask detail. If ``None``, no masking.
                             Function must accept a clip and a reupscaled clip and return a mask.
                             (Default: :py:func:`lvsfunc.scale.descale_detail_mask`).
     :param mask_thr:        Binarization threshold for :py:func:`lvsfunc.scale.descale_detail_mask` (Default: 0.05).
     :param mix_strength:    Merging strength between the descaled and downscaled clip.
-                            Stronger values will make the lineart look closer to the downscaled clip.
+                            Stronger values will make the line-art look closer to the downscaled clip.
                             This can get pretty dangerous very quickly if you use a sharp ``downscaler``!
-    :param show_mask:       Return the ``credit_mask``. If set to `2`, it will return the lineart mask instead.
+    :param show_mask:       Return the ``credit_mask``. If set to `2`, it will return the line-art mask instead.
     :param nnedi3_args:     Additional args to pass to nnedi3.
     :param eedi3_args:      Additional args to pass to eedi3.
 
     :return:                Rescaled clip with a downscaled clip merged with it and credits masked.
+
+    :raises ValueError:     ``mask_thr`` is not between 0.0–1.0.
     """
-    check_variable(clip, "mixed_rescale")
-    assert clip.format
+    assert check_variable(clip, "mixed_rescale")
 
     if not 0 <= mask_thr <= 1:
         raise ValueError(f"mixed_rescale: '`mask_thr` must be between 0.0 and 1.0! Not {mask_thr}!'")
@@ -574,6 +584,7 @@ def mixed_rescale(clip: vs.VideoNode, width: None | int = None, height: int = 72
 
     if isinstance(kernel, str):
         kernel = get_kernel(kernel)()
+
     if isinstance(downscaler, str):
         downscaler = get_kernel(downscaler)()
 
@@ -591,7 +602,7 @@ def mixed_rescale(clip: vs.VideoNode, width: None | int = None, height: int = 72
         downscaled = depth(downscaler(clip_y, width, height), bits)
         downscaler = Catrom()
 
-    merged = core.std.Expr([descaled, downscaled], f'x {mix_strength} * y 1 {mix_strength} - * +')
+    merged = core.akarin.Expr([descaled, downscaled], f'x {mix_strength} * y 1 {mix_strength} - * +')
 
     if isinstance(credit_mask, vs.VideoNode):
         detail_mask = depth(credit_mask, get_depth(clip))
