@@ -1,14 +1,14 @@
 from __future__ import annotations
 
-from enum import IntEnum
-from typing import (Any, Callable, List, Literal, NamedTuple, NoReturn,
-                    Optional, Protocol, Sequence, Tuple, TypeVar, Union)
+from enum import IntEnum, auto
+from typing import (Any, Callable, List, Literal, NamedTuple, NoReturn, Optional, Protocol, Sequence, Tuple, TypeVar,
+                    Union)
 
 import vapoursynth as vs
 
 __all__: List[str] = [
     'Coefs', 'Coordinate', 'CreditMask', 'CURVES', 'CustomScaler', 'Direction', 'F', 'Matrix', 'Position', 'Range',
-    'RegressClips', 'Resolution', 'ScaleAttempt', 'SceneChangeMode', 'Size', 'T', 'VideoProp', 'VSFunction'
+    'RegressClips', 'Resolution', 'ScaleAttempt', 'SceneChangeMode', 'Size', 'T', 'VideoProp', 'VSFunction',
 ]
 
 
@@ -31,6 +31,8 @@ T = TypeVar("T", bound=VideoProp)
 
 
 class Coefs(NamedTuple):
+    """Coefficients for matrix conversions."""
+
     k0: float
     phi: float
     alpha: float
@@ -39,8 +41,11 @@ class Coefs(NamedTuple):
 
 class Coordinate():
     """
-    A positive set of (x, y) coordinates.
+    Positive set of (x, y) coordinates.
+
+    :raises ValueError:     Negative values get passed.
     """
+
     x: int
     y: int
 
@@ -52,40 +57,41 @@ class Coordinate():
 
 
 class Direction(IntEnum):
-    """
-    Enum to simplify direction argument.
-    """
+    """Enum to simplify direction argument."""
+
     HORIZONTAL = 0
     VERTICAL = 1
 
 
 class Resolution(NamedTuple):
-    """ Tuple representing a resolution. """
+    """Tuple representing a resolution."""
 
+    """Width."""
     width: int
-    """ Width. """
 
+    """Height."""
     height: int
-    """ Height. """
 
 
 class ScaleAttempt(NamedTuple):
-    """ Tuple representing a descale attempt. """
+    """Tuple representing a descale attempt."""
 
+    """Descaled frame in native resolution."""
     descaled: vs.VideoNode
-    """ Descaled frame in native resolution. """
 
+    """Descaled frame reupscaled with the same kernel."""
     rescaled: vs.VideoNode
-    """ Descaled frame reupscaled with the same kernel. """
 
+    """The native resolution."""
     resolution: Resolution
-    """ The native resolution. """
 
+    """The subtractive difference between the original and descaled frame."""
     diff: vs.VideoNode
-    """ The subtractive difference between the original and descaled frame. """
 
 
 class SceneChangeMode(IntEnum):
+    """Size type for :py:func:`lvsfunc.render.find_scene_changes`."""
+
     WWXD = 0
     SCXVID = 1
     WWXD_SCXVID_UNION = 2
@@ -93,15 +99,20 @@ class SceneChangeMode(IntEnum):
 
 
 class Position(Coordinate):
-    pass
+    """Position type for :py:class:`lvsfunc.mask.BoundingBox`."""
 
 
 class Size(Coordinate):
-    pass
+    """Size type for :py:class:`lvsfunc.mask.BoundingBox`."""
 
 
 class Matrix(IntEnum):
-    """Matrix coefficients (ITU-T H.265 Table E.5)"""
+    """
+    Matrix coefficients (ITU-T H.265 Table E.5).
+
+    :raises PermissionError:    ``_RESERVED`` gets called.
+    """
+
     RGB = 0
     GBR = 0
     BT709 = 1
@@ -121,21 +132,37 @@ class Matrix(IntEnum):
 
     @property
     def RESERVED(self) -> NoReturn:
+        """Return PermissionError if called."""
         raise PermissionError
 
 
 class VSFunction(Protocol):
+    """VapourSynth function."""
+
     def __call__(self, clip: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:
+        """Call the VapourSynth function."""
+        ...
+
+
+class VSIdxFunction(Protocol):
+    """VapourSynth Indexing/Source function."""
+
+    def __call__(self, path: str, *args: Any, **kwargs: Any) -> vs.VideoNode:
+        """Call the VapourSynth function."""
         ...
 
 
 class Shapes(IntEnum):
+    """Convolution coordinates for :py:func:`lvsfunc.mask.mt_xxpand_multi`."""
+
     RECTANGLE = 0
     ELLIPSE = 1
     LOSANGE = 2
 
 
 class RegressClips(NamedTuple):
+    """Regress clip types for :py:func:`lvsfunc.recon.regress`."""
+
     slope: vs.VideoNode
     intercept: vs.VideoNode
     correlation: vs.VideoNode
@@ -149,3 +176,19 @@ CURVES = Literal[
     vs.TransferCharacteristics.TRANSFER_BT2020_10,
     vs.TransferCharacteristics.TRANSFER_BT2020_12,
 ]
+
+
+class IndexExists(IntEnum):
+    """Check if certain files exist for :py:func:`lvsfunc.misc.source`."""
+
+    PATH_IS_DGI = auto()
+    PATH_IS_IMG = auto()
+    LWI_EXISTS = auto()
+    DGI_EXISTS = auto()
+    NONE = auto()
+
+
+class _VideoNode(vs.VideoNode):
+    """Use for asserting a VideoFormat exists."""
+
+    format: vs.VideoFormat
