@@ -7,11 +7,12 @@ from functools import partial, wraps
 from typing import Any, Callable, Dict, List, Tuple, Type, cast
 
 import vapoursynth as vs
-from vsutil import depth, get_depth, get_subsampling, get_w, get_y
+from typing_extensions import TypeGuard
+from vskernels import Bicubic, Kernel, Matrix, get_kernel, get_matrix
+from vsutil import depth, get_subsampling, get_w, get_y
 
-from .exceptions import (InvalidFormatError, InvalidMatrixError,
-                         VariableFormatError, VariableResolutionError)
-from .types import CURVES, Coefs, F, Matrix, Range, T
+from .exceptions import InvalidFormatError, InvalidMatrixError, VariableFormatError, VariableResolutionError
+from .types import CURVES, Coefs, F, Range, T, _VideoNode
 
 core = vs.core
 
@@ -20,20 +21,15 @@ __all__: List[str] = [
     'allow_variable',
     'check_variable',
     'chroma_injector',
-    'clamp_values',
     'colored_clips',
     'force_mod',
     'frames_since_bookmark',
     'get_coefs',
     'get_matrix_curve',
-    'get_matrix',
-    'get_neutral_value',
     'get_prop',
     'load_bookmarks',
     'normalize_ranges',
     'padder',
-    'pick_removegrain',
-    'pick_repair',
     'quick_resample',
     'replace_ranges', 'rfs',
     'scale_peak',
@@ -45,16 +41,22 @@ def quick_resample(clip: vs.VideoNode,
                    function: Callable[[vs.VideoNode], vs.VideoNode]
                    ) -> vs.VideoNode:
     """
-    A function to quickly resample to 32/16/8 bit and back to the original depth in a one-liner.
+    Quickly resample to 32/16/8 bit and back to the original depth in a one-liner.
+
+    .. warning:
+        This function will be either reworked or removed in a future version!
+
     Useful for filters that only work in 16 bit or lower when you're working in float.
 
-    :param clip:      Input clip
-    :param function:  Filter to run after resampling (accepts and returns clip)
+    :param clip:        Clip to process.
+    :param function:    Filter to run after resampling (accepts and returns clip).
 
-    :return:          Filtered clip in original depth
+    :return:            Filtered clip in original depth.
     """
-    check_variable_format(clip, "quick_resample")
-    assert clip.format
+    warnings.warn("quick_resample: 'This function will be either reworked or removed in a future version!'",
+                  FutureWarning)
+
+    assert check_variable_format(clip, "quick_resample")
 
     try:  # Excepts all generic because >plugin/script writers being consistent >_>
         dither = depth(clip, 32)
@@ -70,56 +72,18 @@ def quick_resample(clip: vs.VideoNode,
     return depth(filtered, clip.format.bits_per_sample)
 
 
-def pick_repair(clip: vs.VideoNode) -> Callable[..., vs.VideoNode]:
-    """
-    Returns rgvs.Repair if the clip is 16 bit or lower, else rgsf.Repair.
-    This is done because rgvs doesn't work with float, but rgsf does for whatever reason.
-
-    Dependencies: rgsf
-
-    :param clip: Input clip
-
-    :return:     Appropriate repair function for input clip's depth
-    """
-    check_variable_format(clip, "pick_repair")
-    assert clip.format
-
-    is_float = clip.format.sample_type == vs.FLOAT
-
-    return core.rgvs.Repair if is_float else core.rgsf.Repair
-
-
-def pick_removegrain(clip: vs.VideoNode) -> Callable[..., vs.VideoNode]:
-    """
-    Returns rgvs.RemoveGrain if the clip is 16 bit or lower, else rgsf.RemoveGrain.
-    This is done because rgvs doesn't work with float, but rgsf does for whatever reason.
-
-    Dependencies:
-
-    * RGSF
-
-    :param clip: Input clip
-
-    :return:     Appropriate RemoveGrain function for input clip's depth
-    """
-    check_variable_format(clip, "pick_removegrain")
-    assert clip.format
-
-    is_float = clip.format.sample_type == vs.FLOAT
-
-    return core.rgvs.RemoveGrain if is_float else core.rgsf.RemoveGrain
-
-
 def get_prop(frame: vs.VideoFrame, key: str, t: Type[T]) -> T:
     """
-    Gets FrameProp ``prop`` from frame ``frame`` with expected type ``t``
-    to satisfy the type checker.
+    Get FrameProp ``prop`` from frame ``frame`` with expected type ``t`` to satisfy the type checker.
 
-    :param frame:   Frame containing props
-    :param key:     Prop to get
-    :param t:       Type of prop
+    :param frame:           Frame containing props.
+    :param key:             Prop to get.
+    :param t:               Type of prop.
 
-    :return:        frame.prop[key]
+    :return:                frame.prop[key].
+
+    :raises KeyError:       ``key`` is not found in props.
+    :raises ValueError:     Returns a prop of the wrong type.
     """
     try:
         prop = frame.props[key]
@@ -133,13 +97,14 @@ def get_prop(frame: vs.VideoFrame, key: str, t: Type[T]) -> T:
 
 
 def normalize_ranges(clip: vs.VideoNode, ranges: Range | List[Range]) -> List[Tuple[int, int]]:
-    """
-    Normalize ``Range``\\(s) to a list of inclusive positive integer ranges.
+    r"""
+    Normalize :py:func:`lvsfunc.types.Range`\(s) to a list of inclusive positive integer ranges.
 
-    :param clip:   Reference clip used for length.
-    :param ranges: Single ``Range`` or list of ``Range``\\s.
+    :param clip:        Reference clip used for length.
+    :param ranges:      Single :py:class:`lvsfunc.types.Range`,
+                        or a list of :py:class:`lvsfunc.types.Range`\(s).
 
-    :return:       List of inclusive positive ranges.
+    :return:            List of inclusive positive ranges.
     """
     ranges = ranges if isinstance(ranges, list) else [ranges]
 
@@ -169,10 +134,14 @@ def normalize_ranges(clip: vs.VideoNode, ranges: Range | List[Range]) -> List[Tu
 def replace_ranges(clip_a: vs.VideoNode,
                    clip_b: vs.VideoNode,
                    ranges: Range | List[Range] | None,
+                   exclusive: bool = False,
                    use_plugin: bool = True) -> vs.VideoNode:
     """
-    A replacement for ReplaceFramesSimple that uses ints and tuples rather than a string.
-    Frame ranges are inclusive. Optionally strings can still be used.
+    Remaps frame indices in a clip using ints and tuples rather than a string.
+
+    Frame ranges are inclusive. This behaviour can be changed by setting `exclusive=True`.
+
+    If you're trying to splice in clips, it's recommended you use `vsutil.insert_clip` instead.
 
     This function will try to call the `VapourSynth-RemapFrames` plugin before doing any of its own processing.
     This should come with a speed boost, so it's recommended you install it.
@@ -186,44 +155,63 @@ def replace_ranges(clip_a: vs.VideoNode,
         * ``replace_ranges(black, white, [(200, -1)])``: replace 200 until the end with ``white``,
           leaving 1 frame of ``black``
 
-    Dependencies: VapourSynth-RemapFrames
+    Alias for this function is ``lvsfunc.rfs``.
 
-    :param clip_a:      Original clip
-    :param clip_b:      Replacement clip
-    :param ranges:      Ranges to replace clip_a (original clip) with clip_b (replacement clip).
+    Dependencies:
 
-                        Integer values in the list indicate single frames,
+    * `VapourSynth-RemapFrames <https://github.com/Irrational-Encoding-Wizardry/Vapoursynth-RemapFrames>`_
 
-                        Tuple values indicate inclusive ranges.
+    :param clip_a:          Original clip.
+    :param clip_b:          Replacement clip.
+    :param ranges:          Ranges to replace clip_a (original clip) with clip_b (replacement clip).
 
-                        Negative integer values will be wrapped around based on clip_b's length.
+                            Integer values in the list indicate single frames,
 
-                        None values are context dependent:
+                            Tuple values indicate inclusive ranges.
 
-                            * None provided as sole value to ranges: no-op
-                            * Single None value in list: Last frame in clip_b
-                            * None as first value of tuple: 0
-                            * None as second value of tuple: Last frame in clip_b
-    :param use_plugin:  Use the ReplaceFramesSimple plugin for the rfs call.
+                            Negative integer values will be wrapped around based on clip_b's length.
 
-    :return:           Clip with ranges from clip_a replaced with clip_b
+                            None values are context dependent:
+
+                                * None provided as sole value to ranges: no-op
+                                * Single None value in list: Last frame in clip_b
+                                * None as first value of tuple: 0
+                                * None as second value of tuple: Last frame in clip_b
+    :param exclusive:       Use exclusive ranges (Default: False).
+    :param use_plugin:      Use the ReplaceFramesSimple plugin for the rfs call (Default: True).
+
+    :return:                Clip with ranges from clip_a replaced with clip_b.
+
+    :raises ValueError:     A string is passed instead of a list of ranges.
     """
-    if ranges is None:
+    if ranges != 0 and not ranges:
         return clip_a
 
+    if isinstance(ranges, str):  # type:ignore[unreachable]
+        raise ValueError("replace_ranges: 'This function does not take strings! Please use a list of tuples or ints!")
+
+    if clip_a.num_frames != clip_b.num_frames:
+        warnings.warn("replace_ranges: "
+                      f"'The number of frames ({clip_a.num_frames} vs. {clip_b.num_frames}) do not match! "
+                      "The function will still work, but you may run into unintended errors with the output clip!'")
+
     nranges = normalize_ranges(clip_b, ranges)
 
     if use_plugin and hasattr(core, 'remap'):
-        return core.remap.ReplaceFramesSimple(clip_a, clip_b, mappings=' '.join(f'[{s} {e}]' for s, e in nranges))
+        return core.remap.ReplaceFramesSimple(
+            clip_a, clip_b, mismatch=True,
+            mappings=' '.join(f'[{s} {e + (exclusive if s != e else 0)}]' for s, e in nranges)
+        )
 
     out = clip_a
+    shift = 1 + exclusive
 
     for start, end in nranges:
-        tmp = clip_b[start:end + 1]
+        tmp = clip_b[start:end + shift]
         if start != 0:
             tmp = out[: start] + tmp
         if end < out.num_frames - 1:
-            tmp = tmp + out[end + 1:]
+            tmp = tmp + out[end + shift:]
         out = tmp
 
     return out
@@ -233,15 +221,16 @@ def scale_thresh(thresh: float, clip: vs.VideoNode, assume: int | None = None) -
     """
     Scale binarization thresholds from float to int.
 
-    :param thresh: Threshold [0, 1]. If greater than 1, assumed to be in native clip range
-    :param clip:   Clip to scale to
-    :param assume: Assume input is this depth when given input >1. If ``None``\\, assume ``clip``\\'s format.
-                   (Default: None)
+    :param thresh:          Threshold [0, 1]. If greater than 1, assumed to be in native clip range.
+    :param clip:            Clip to scale to.
+    :param assume:          | Assume input is this depth when given input > 1.
+                            | If ``None``, assume ``clip``'s format (Default: None).
 
-    :return:       Threshold scaled to [0, 2^clip.depth - 1] (if vs.INTEGER)
+    :return:                Threshold scaled to [0, 2^clip.depth - 1] (if vs.INTEGER).
+
+    :raises ValueError:     Thresholds are negative.
     """
-    check_variable_format(clip, "scale_thresh")
-    assert clip.format
+    assert check_variable_format(clip, "scale_thresh")
 
     if thresh < 0:
         raise ValueError("scale_thresh: 'Thresholds must be positive!'")
@@ -253,64 +242,42 @@ def scale_thresh(thresh: float, clip: vs.VideoNode, assume: int | None = None) -
 
 
 def scale_peak(value: float, peak: float) -> float:
-    """
-    Full-range scale function that scales a value from [0, 255] to [0, peak]
-    """
+    """Full-range scale function that scales a value from [0, 255] to [0, peak]."""
     return value * peak / 255
 
 
 def force_mod(x: float, mod: int = 4) -> int:
     """
     Force output to fit a specific MOD.
+
     Minimum returned value will always be mod².
     """
     return mod ** 2 if x < mod ** 2 else int(x / mod + 0.5) * mod
 
 
-def clamp_values(x: float, max_val: float, min_val: float) -> float:
-    """
-    Forcibly clamps the given value x to a max and/or min value.
-    """
-    return min_val if x < min_val else max_val if x > max_val else x
-
-
-def get_neutral_value(clip: vs.VideoNode, chroma: bool = False) -> float:
-    """
-    Taken from vsutil. This isn't in any new versions yet, so mypy complains.
-    Will remove once vsutil does another version bump.
-
-    Returns the neutral value for the combination
-    of the plane type and bit depth/type of the clip as float.
-
-    :param clip:        Input clip.
-    :param chroma:      Whether to get luma or chroma plane value
-
-    :return:            Neutral value.
-    """
-    check_variable_format(clip, "get_neutral_value")
-    assert clip.format
-
-    is_float = clip.format.sample_type == vs.FLOAT
-
-    return (0. if chroma else 0.5) if is_float else float(1 << (get_depth(clip) - 1))
-
-
 def padder(clip: vs.VideoNode,
            left: int = 32, right: int = 32,
            top: int = 32, bottom: int = 32) -> vs.VideoNode:
     """
-    Pads out the pixels on the side by the given amount of pixels.
+    Pad out the pixels on the side by the given amount of pixels.
+
     For a 4:2:0 clip, the output must be an even resolution.
 
-    :param clip:        Input clip
-    :param left:        Padding added to the left side of the clip
-    :param right:       Padding added to the right side of the clip
-    :param top:         Padding added to the top side of the clip
-    :param bottom:      Padding added to the bottom side of the clip
+    Dependencies:
 
-    :return:            Padded clip
+    * `VapourSynth-fillborders <https://github.com/dubhater/vapoursynth-fillborders>`_
+
+    :param clip:        Clip to process.
+    :param left:        Padding added to the left side of the clip.
+    :param right:       Padding added to the right side of the clip.
+    :param top:         Padding added to the top side of the clip.
+    :param bottom:      Padding added to the bottom side of the clip.
+
+    :return:            Padded clip.
+
+    :raises ValueError: A non-even resolution is passed on a YUV420 clip.
     """
-    check_variable(clip, "padder")
+    assert check_variable(clip, "padder")
 
     width = clip.width+left+right
     height = clip.height+top+bottom
@@ -325,7 +292,7 @@ def padder(clip: vs.VideoNode,
 
 
 def get_coefs(curve: vs.TransferCharacteristics) -> Coefs:
-    """Returns transfer coefs."""
+    """Return transfer coefs."""
     srgb = Coefs(0.04045, 12.92, 0.055, 2.4)
     bt709 = Coefs(0.08145, 4.5, 0.0993, 2.22222)
     smpte240m = Coefs(0.0912, 4.0, 0.1115, 2.22222)
@@ -343,75 +310,65 @@ def get_coefs(curve: vs.TransferCharacteristics) -> Coefs:
     return gamma_linear_map[curve]
 
 
-def check_variable_format(clip: vs.VideoNode, function: str) -> None:
-    """Check for variable format, and return an error if found."""
+def check_variable_format(clip: vs.VideoNode, function: str) -> TypeGuard[_VideoNode]:
+    """
+    Check for variable format and return an error if found.
+
+    :raises VariableFormatError:    The clip is of a variable format.
+    """
     if clip.format is None:
         raise VariableFormatError(function)
+    return True
 
 
 def check_variable_resolution(clip: vs.VideoNode, function: str) -> None:
-    """Check for variable width or height, and return an error if found."""
+    """
+    Check for variable width or height and return an error if found.
+
+    :raises VariableResolutionError:    The clip has a variable resolution.
+    """
     if 0 in (clip.width, clip.height):
         raise VariableResolutionError(function)
 
 
-def check_variable(clip: vs.VideoNode, function: str) -> None:
-    """Check for variable format and a variable resolution, and return an error if found."""
+def check_variable(clip: vs.VideoNode, function: str) -> TypeGuard[_VideoNode]:
+    """
+    Check for variable format and a variable resolution and return an error if found.
+
+    :raises VariableFormatError:        The clip is of a variable format.
+    :raises VariableResolutionError:    The clip has a variable resolution.
+    """
     check_variable_format(clip, function)
     check_variable_resolution(clip, function)
+    return True
 
 
-def get_matrix(clip: vs.VideoNode, return_matrix: bool = False) -> Matrix | int:
+def get_matrix_curve(matrix: Matrix) -> CURVES:
     """
-    Helper function to get the matrix for a clip.
+    Return the matrix curve based on a given ``matrix``.
 
-    :param clip:            Input clip
-    :param return_matrix:   Returns a Matrix instead of an int.
-                            Set to False by default for backwards compatibility.
-
-    :return:                Value representing a matrix
+    :raises InvalidMatrixError:     An invalid ``matrix`` is passed.
     """
-    check_variable_format(clip, "get_matrix")
-    assert clip.format
-
-    if not return_matrix:
-        warnings.warn("get_matrix: '`return_matrix=True` will be set to default in a future commit! "
-                      "Make sure you update your functions to work with `Matrix` objects!'")
-
-    frame = clip.get_frame(0)
-    w, h = frame.width, frame.height
-
-    if frame.format.color_family == vs.RGB:
-        return Matrix.RGB if return_matrix else 0
-    elif w <= 1024 and h <= 576:
-        return Matrix.BT470BG if return_matrix else 5
-    elif w <= 2048 and h <= 1536:
-        return Matrix.BT709 if return_matrix else 1
-    return Matrix.BT2020NC if return_matrix else 9
-
-
-def get_matrix_curve(matrix: int) -> CURVES:
-    """Returns a matrix curve based on a given `matrix`."""
     match matrix:
-        case 1: return vs.TransferCharacteristics.TRANSFER_BT709
-        case 5 | 6: return vs.TransferCharacteristics.TRANSFER_BT601
-        case 7: return vs.TransferCharacteristics.TRANSFER_ST240_M
-        case 13: return vs.TransferCharacteristics.TRANSFER_IEC_61966_2_1
-        case 14: return vs.TransferCharacteristics.TRANSFER_BT2020_10
-        case 15: return vs.TransferCharacteristics.TRANSFER_BT2020_12
+        case Matrix.BT709: return vs.TransferCharacteristics.TRANSFER_BT709
+        case Matrix.BT470BG | Matrix.SMPTE170M: return vs.TransferCharacteristics.TRANSFER_BT601
+        case Matrix.SMPTE240M: return vs.TransferCharacteristics.TRANSFER_ST240_M
+        case Matrix.CHROMA_DERIVED_C: return vs.TransferCharacteristics.TRANSFER_IEC_61966_2_1
+        case Matrix.ICTCP: return vs.TransferCharacteristics.TRANSFER_BT2020_10
+        # case 15: return vs.TransferCharacteristics.TRANSFER_BT2020_12
         case _: raise InvalidMatrixError("get_matrix_curve", message="{func}: 'An invalid matrix value was passed!'")
 
 
 def load_bookmarks(bookmark_path: str) -> List[int]:
     """
-    VSEdit bookmark loader.
+    Load VSEdit bookmarks.
 
     load_bookmarks(os.path.basename(__file__)+".bookmarks")
     will load the VSEdit bookmarks for the current Vapoursynth script.
 
-    :param bookmark_path:  Path to bookmarks file
+    :param bookmark_path:  Path to bookmarks file.
 
-    :return:               A list of bookmarked frames
+    :return:               A list of bookmarked frames.
     """
     with open(bookmark_path) as f:
         bookmarks = [int(i) for i in f.read().split(", ")]
@@ -424,13 +381,14 @@ def load_bookmarks(bookmark_path: str) -> List[int]:
 
 def frames_since_bookmark(clip: vs.VideoNode, bookmarks: List[int]) -> vs.VideoNode:
     """
-    Displays frames since last bookmark to create easily reusable scenefiltering.
+    Display frames since last bookmark to create easily reusable scenefiltering.
+
     Can be used in tandem with :py:func:`lvsfunc.misc.load_bookmarks` to import VSEdit bookmarks.
 
-    :param clip:        Input clip
-    :param bookmarks:   A list of bookmarks
+    :param clip:        Clip to process.
+    :param bookmarks:   A list of bookmarks.
 
-    :return:            Clip with bookmarked frames
+    :return:            Clip with bookmarked frames.
     """
     def _frames_since_bookmark(n: int, clip: vs.VideoNode, bookmarks: List[int]) -> vs.VideoNode:
         for i, bookmark in enumerate(bookmarks):
@@ -448,21 +406,34 @@ def frames_since_bookmark(clip: vs.VideoNode, bookmarks: List[int]) -> vs.VideoN
 
 def chroma_injector(func: F) -> F:
     """
-    Decorator allowing injection of reference chroma into a function which
-    would normally only receive luma, such as an upscaler passed to
-    :py:func:`lvsfunc.scale.descale`. The chroma is resampled to the input
-    clip's width, height, and pixel format, shuffled to YUV444PX, then passed
-    to the function. Luma is then extracted from the function result and
-    returned. The first argument of the function is assumed to be the luma
-    source. This works with variable resolution and may work with variable
-    format, however the latter is wholly untested and likely a bad idea in
-    every conceivable use case.
+    Inject reference chroma.
 
-    :param func:        Function to call with injected chroma
+    This is a function decorator. That means it must be called above a function. For example:
 
-    :return:            Decorated function
+    .. code-block:: py
+
+        @chroma_injector()
+        def function(clip: vs.VideoNode) -> vs.VideoNode:
+            ...
+
+    This can be used to inject reference chroma into a function which would normally
+    only receive luma, such as an upscaler passed to :py:func:`lvsfunc.scale.descale`.
+
+    The chroma is resampled to the input clip's width, height, and pixel format,
+    shuffled to YUV444PX, then passed to the function.
+    Luma is then extracted from the function result and returned.
+
+    The first argument of the function is assumed to be the luma source.
+    This works with variable resolution and may work with variable format,
+    however the latter is wholly untested and likely a bad idea in every conceivable use case.
+
+    :param func:                    Function to call with injected chroma.
+
+    :return:                        Decorated function.
+
+    :raises InvalidFormatError:     Input clip is not YUV or GRAY.
+    :raises InvalidFormatError:     Output clip is not YUV or GRAY.
     """
-
     @wraps(func)
     def inner(_chroma: vs.VideoNode, clip: vs.VideoNode, *args: Any,
               **kwargs: Any) -> vs.VideoNode:
@@ -506,7 +477,8 @@ def chroma_injector(func: F) -> F:
 
         if result.format is not None:
             if result.format.color_family not in (vs.GRAY, vs.YUV):
-                raise ValueError("chroma_injector: can only decorate function with YUV and/or GRAY format return!")
+                raise InvalidFormatError("chroma_injector",
+                                         "{func}: 'can only decorate function with YUV and/or GRAY format return!'")
 
             if result.format.color_family == vs.GRAY:
                 return result
@@ -526,28 +498,32 @@ def colored_clips(amount: int,
                   seed: bytearray | bytes | float | str | None = None,
                   **kwargs: Any
                   ) -> List[vs.VideoNode]:
-    """
-    Returns a list of BlankClips with unique colors in sequential or random order.
+    r"""
+    Return a list of BlankClips with unique colors in sequential or random order.
+
     The colors will be evenly spaced by hue in the HSL colorspace.
 
     Useful maybe for comparison functions or just for getting multiple uniquely colored BlankClips for testing purposes.
 
     Will always return a pure red clip in the list as this is the RGB equivalent of the lowest HSL hue possible (0).
 
-    Written by Dave <orangechannel@pm.me>.
+    Written by `Dave <https://github.com/OrangeChannel>`_.
 
-    :param amount:  Number of ``vapoursynth.VideoNode``\\s to return
-    :param max_hue: Maximum hue (0 < hue <= 360) in degrees to generate colors from (uses the HSL color model).
-                    Setting this higher than ``315`` will result in the clip colors looping back towards red
-                    and is not recommended for visually distinct colors.
-                    If the `amount` of clips is higher than the `max_hue` expect there to be identical
-                    or visually similar colored clips returned (Default: 300)
-    :param rand:    Randomizes order of the returned list (Default: True)
-    :param seed:    Bytes-like object passed to ``random.seed`` which allows for consistent randomized order
-                    of the resulting clips (Default: None)
-    :param kwargs:  Arguments passed to ``vapoursynth.core.std.BlankClip`` (Default: keep=1)
+    :param amount:          Number of VideoNodes to return.
+    :param max_hue:         Maximum hue (0 < hue <= 360) in degrees to generate colors from (uses the HSL color model).
+                            Setting this higher than ``315`` will result in the clip colors looping back towards red
+                            and is not recommended for visually distinct colors.
+                            If the `amount` of clips is higher than the `max_hue` expect there to be identical
+                            or visually similar colored clips returned (Default: 300)
+    :param rand:            Randomizes order of the returned list (Default: True).
+    :param seed:            Bytes-like object passed to ``random.seed`` which allows for consistent randomized order.
+                            of the resulting clips (Default: None)
+    :param kwargs:          Arguments passed to :py:func:`vapoursynth.core.std.BlankClip` (Default: keep=1).
 
-    :return:        List of uniquely colored clips in sequential or random order.
+    :return:                List of uniquely colored clips in sequential or random order.
+
+    :raises ValueError:     ``amount`` is less than 2.
+    :raises ValueError:     ``max_hue`` is not between 0–360.
     """
     if amount < 2:
         raise ValueError("colored_clips: `amount` must be at least 2!")
@@ -573,18 +549,28 @@ def allow_variable(width: int | None = None, height: int | None = None,
                    format: int | None = None
                    ) -> Callable[[Callable[..., vs.VideoNode]], Callable[..., vs.VideoNode]]:
     """
-    Decorator allowing a variable-res and/or variable-format clip to be passed
-    to a function that otherwise would not be able to accept it. Implemented by
-    FrameEvaling and resizing the clip to each frame. Does not work when the
-    function needs to return a different format unless an output format is
-    specified. As such, this decorator must be called as a function when used
-    (e.g. ``@allow_variable()`` or ``@allow_variable(format=vs.GRAY16)``). If
-    the provided clip is variable format, no output format is required to be
-    specified.
+    Allow a variable-res and/or variable-format clip to be passed to a function.
 
-    :param width:       Output clip width
-    :param height:      Output clip height
-    :param format:      Output clip format
+    This is a function decorator. That means it must be called above a function. For example:
+
+    .. code-block:: py
+
+        @allow_variable()
+        def function(clip: vs.VideoNode) -> vs.VideoNode:
+            ...
+
+    This can be used on functions that otherwise would not be able to accept it.
+    Implemented by FrameEvaling and resizing the clip to each frame.
+
+    Does not work when the function needs to return a different format unless an output format is specified.
+    As such, this decorator must be called as a function when used (e.g. ``@allow_variable()``
+    or ``@allow_variable(format=vs.GRAY16)``).
+
+    If the provided clip is variable format, no output format is required to be specified.
+
+    :param width:       Output clip width.
+    :param height:      Output clip height.
+    :param format:      Output clip format.
 
     :return:            Function decorator for the given output format.
     """
@@ -607,5 +593,47 @@ def allow_variable(width: int | None = None, height: int | None = None,
     return inner
 
 
+def match_clip(clip: vs.VideoNode, ref: vs.VideoNode,
+               dimensions: bool = True, vformat: bool = True,
+               matrices: bool = True, length: bool = False,
+               kernel: Kernel | str = Bicubic(b=0, c=1/2)) -> vs.VideoNode:
+    """
+    Try matching the given clip's format with the reference clip's.
+
+    :param clip:        Clip to process.
+    :param ref:         Reference clip.
+    :param dimensions:  Match video dimensions (Default: True).
+    :param vformat:     Match video formats (Default: True).
+    :param matrices:    Match matrix/transfer/primaries (Default: True).
+    :param length:      Match clip length (Default: False).
+    :param kernel:      py:class:`vskernels.Kernel` object used for the format conversion.
+                        This can also be the string name of the kernel
+                        (Default: py:class:`vskernels.Bicubic(b=0, c=1/2)`).
+
+    :return:            Clip that matches the ref clip in format.
+    """
+    assert check_variable(clip, "match_clip")
+    assert check_variable(ref, "match_clip")
+
+    if isinstance(kernel, str):
+        kernel = get_kernel(kernel)()
+
+    clip = clip * ref.num_frames if length else clip
+    clip = kernel.scale(clip, ref.width, ref.height) if dimensions else clip
+
+    if vformat:
+        clip = kernel.resample(clip, format=ref.format, matrix=get_matrix(ref))
+
+    if matrices:
+        ref_frame = ref.get_frame(0)
+
+        clip = clip.std.SetFrameProps(
+            _Matrix=get_prop(ref_frame, '_Matrix', int),
+            _Transfer=get_prop(ref_frame, '_Transfer', int),
+            _Primaries=get_prop(ref_frame, '_Primaries', int))
+
+    return clip.std.AssumeFPS(fpsnum=ref.fps.numerator, fpsden=ref.fps.denominator)
+
+
 # Aliases
 rfs = replace_ranges
